@@ -1,7 +1,7 @@
 /**
  * <copyright> 
  *
- * Copyright (c) 2002-2005 IBM Corporation and others.
+ * Copyright (c) 2002-2006 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: GenModel.java,v 1.26 2005/06/28 23:49:12 marcelop Exp $
+ * $Id: GenModel.java,v 1.37 2006/05/01 10:28:29 davidms Exp $
  */
 package org.eclipse.emf.codegen.ecore.genmodel;
 
@@ -25,8 +25,11 @@ import java.util.Set;
 import org.eclipse.core.runtime.IStatus;
 
 import org.eclipse.emf.codegen.jet.JETEmitter;
-import org.eclipse.emf.codegen.jmerge.JControlModel;
+import org.eclipse.emf.codegen.merge.java.JControlModel;
+import org.eclipse.emf.codegen.util.ImportManager;
+import org.eclipse.emf.common.util.Diagnostic;
 import org.eclipse.emf.common.util.EList;
+import org.eclipse.emf.ecore.EClass;
 import org.eclipse.emf.ecore.EPackage;
 import org.eclipse.emf.ecore.util.ExtendedMetaData;
 import org.eclipse.jdt.core.formatter.CodeFormatter;
@@ -67,6 +70,9 @@ import org.eclipse.jdt.core.formatter.CodeFormatter;
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getRootExtendsClass <em>Root Extends Class</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getRootImplementsInterface <em>Root Implements Interface</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressEMFTypes <em>Suppress EMF Types</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressEMFMetaData <em>Suppress EMF Meta Data</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressEMFModelTags <em>Suppress EMF Model Tags</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressInterfaces <em>Suppress Interfaces</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFeatureMapWrapperInterface <em>Feature Map Wrapper Interface</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFeatureMapWrapperInternalInterface <em>Feature Map Wrapper Internal Interface</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFeatureMapWrapperClass <em>Feature Map Wrapper Class</em>}</li>
@@ -80,6 +86,14 @@ import org.eclipse.jdt.core.formatter.CodeFormatter;
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getBooleanFlagsReservedBits <em>Boolean Flags Reserved Bits</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getImporterID <em>Importer ID</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isBundleManifest <em>Bundle Manifest</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFeatureDelegation <em>Feature Delegation</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isContainmentProxies <em>Containment Proxies</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isMinimalReflectiveMethods <em>Minimal Reflective Methods</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressContainment <em>Suppress Containment</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressNotification <em>Suppress Notification</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isArrayAccessors <em>Array Accessors</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressUnsettable <em>Suppress Unsettable</em>}</li>
+ *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFacadeHelperClass <em>Facade Helper Class</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getGenPackages <em>Gen Packages</em>}</li>
  *   <li>{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getUsedGenPackages <em>Used Gen Packages</em>}</li>
  * </ul>
@@ -89,7 +103,8 @@ import org.eclipse.jdt.core.formatter.CodeFormatter;
  * @model
  * @generated
  */
-public interface GenModel extends GenBase{
+public interface GenModel extends GenBase
+{
   /**
    * Returns the value of the '<em><b>Copyright Text</b></em>' attribute.
    * <!-- begin-user-doc -->
@@ -864,6 +879,84 @@ public interface GenModel extends GenBase{
    */
   void setSuppressEMFTypes(boolean value);
 
+  /**
+   * Returns the value of the '<em><b>Suppress EMF Meta Data</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress EMF Meta Data</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress EMF Meta Data</em>' attribute.
+   * @see #setSuppressEMFMetaData(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressEMFMetaData()
+   * @model
+   * @generated
+   */
+  boolean isSuppressEMFMetaData();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressEMFMetaData <em>Suppress EMF Meta Data</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress EMF Meta Data</em>' attribute.
+   * @see #isSuppressEMFMetaData()
+   * @generated
+   */
+  void setSuppressEMFMetaData(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Suppress EMF Model Tags</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress EMF Model Tags</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress EMF Model Tags</em>' attribute.
+   * @see #setSuppressEMFModelTags(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressEMFModelTags()
+   * @model
+   * @generated
+   */
+  boolean isSuppressEMFModelTags();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressEMFModelTags <em>Suppress EMF Model Tags</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress EMF Model Tags</em>' attribute.
+   * @see #isSuppressEMFModelTags()
+   * @generated
+   */
+  void setSuppressEMFModelTags(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Suppress Interfaces</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress Interfaces</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress Interfaces</em>' attribute.
+   * @see #setSuppressInterfaces(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressInterfaces()
+   * @model
+   * @generated
+   */
+  boolean isSuppressInterfaces();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressInterfaces <em>Suppress Interfaces</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress Interfaces</em>' attribute.
+   * @see #isSuppressInterfaces()
+   * @generated
+   */
+  void setSuppressInterfaces(boolean value);
+
   /**
    * Returns the value of the '<em><b>Feature Map Wrapper Interface</b></em>' attribute.
    * <!-- begin-user-doc -->
@@ -1005,7 +1098,7 @@ public interface GenModel extends GenBase{
    * @return the value of the '<em>Reflective Delegation</em>' attribute.
    * @see #setReflectiveDelegation(boolean)
    * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_ReflectiveDelegation()
-   * @model
+   * @model volatile="true" derived="true"
    * @generated
    */
   boolean isReflectiveDelegation();
@@ -1013,6 +1106,7 @@ public interface GenModel extends GenBase{
   /**
    * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isReflectiveDelegation <em>Reflective Delegation</em>}' attribute.
    * <!-- begin-user-doc -->
+   * @deprecated In 2.2. Please use {@link #setFeatureDelegation} instead.
    * <!-- end-user-doc -->
    * @param value the new value of the '<em>Reflective Delegation</em>' attribute.
    * @see #isReflectiveDelegation()
@@ -1257,6 +1351,222 @@ public interface GenModel extends GenBase{
    */
   void setBundleManifest(boolean value);
 
+  /**
+   * Returns the value of the '<em><b>Feature Delegation</b></em>' attribute.
+   * The literals are from the enumeration {@link org.eclipse.emf.codegen.ecore.genmodel.GenDelegationKind}.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Feature Delegation</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Feature Delegation</em>' attribute.
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenDelegationKind
+   * @see #setFeatureDelegation(GenDelegationKind)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_FeatureDelegation()
+   * @model
+   * @generated
+   */
+  GenDelegationKind getFeatureDelegation();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFeatureDelegation <em>Feature Delegation</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Feature Delegation</em>' attribute.
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenDelegationKind
+   * @see #getFeatureDelegation()
+   * @generated
+   */
+  void setFeatureDelegation(GenDelegationKind value);
+
+  /**
+   * Returns the value of the '<em><b>Containment Proxies</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Containment Proxies</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Containment Proxies</em>' attribute.
+   * @see #setContainmentProxies(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_ContainmentProxies()
+   * @model
+   * @generated
+   */
+  boolean isContainmentProxies();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isContainmentProxies <em>Containment Proxies</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Containment Proxies</em>' attribute.
+   * @see #isContainmentProxies()
+   * @generated
+   */
+  void setContainmentProxies(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Minimal Reflective Methods</b></em>' attribute.
+   * The default value is <code>"true"</code>.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Minimal Reflective Methods</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Minimal Reflective Methods</em>' attribute.
+   * @see #setMinimalReflectiveMethods(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_MinimalReflectiveMethods()
+   * @model default="true"
+   * @generated
+   */
+  boolean isMinimalReflectiveMethods();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isMinimalReflectiveMethods <em>Minimal Reflective Methods</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Minimal Reflective Methods</em>' attribute.
+   * @see #isMinimalReflectiveMethods()
+   * @generated
+   */
+  void setMinimalReflectiveMethods(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Suppress Containment</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress Containment</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress Containment</em>' attribute.
+   * @see #setSuppressContainment(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressContainment()
+   * @model
+   * @generated
+   */
+  boolean isSuppressContainment();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressContainment <em>Suppress Containment</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress Containment</em>' attribute.
+   * @see #isSuppressContainment()
+   * @generated
+   */
+  void setSuppressContainment(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Suppress Notification</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress Notification</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress Notification</em>' attribute.
+   * @see #setSuppressNotification(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressNotification()
+   * @model
+   * @generated
+   */
+  boolean isSuppressNotification();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressNotification <em>Suppress Notification</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress Notification</em>' attribute.
+   * @see #isSuppressNotification()
+   * @generated
+   */
+  void setSuppressNotification(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Array Accessors</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Array Accessors</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Array Accessors</em>' attribute.
+   * @see #setArrayAccessors(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_ArrayAccessors()
+   * @model
+   * @generated
+   */
+  boolean isArrayAccessors();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isArrayAccessors <em>Array Accessors</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Array Accessors</em>' attribute.
+   * @see #isArrayAccessors()
+   * @generated
+   */
+  void setArrayAccessors(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Suppress Unsettable</b></em>' attribute.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Suppress Unsettable</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * @return the value of the '<em>Suppress Unsettable</em>' attribute.
+   * @see #setSuppressUnsettable(boolean)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_SuppressUnsettable()
+   * @model
+   * @generated
+   */
+  boolean isSuppressUnsettable();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#isSuppressUnsettable <em>Suppress Unsettable</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Suppress Unsettable</em>' attribute.
+   * @see #isSuppressUnsettable()
+   * @generated
+   */
+  void setSuppressUnsettable(boolean value);
+
+  /**
+   * Returns the value of the '<em><b>Facade Helper Class</b></em>' attribute.
+   * The default value is <code>"org.eclipse.emf.codegen.merge.java.facade.jdom.JDOMFacadeHelper"</code>.
+   * <!-- begin-user-doc -->
+   * <p>
+   * If the meaning of the '<em>Facade Helper Class</em>' attribute isn't clear,
+   * there really should be more of a description here...
+   * </p>
+   * <!-- end-user-doc -->
+   * <!-- begin-model-doc -->
+   * The default value must be always equals to JMerger.DEFAULT_FACADE_HELPER_CLASS.
+   * <!-- end-model-doc -->
+   * @return the value of the '<em>Facade Helper Class</em>' attribute.
+   * @see #setFacadeHelperClass(String)
+   * @see org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage#getGenModel_FacadeHelperClass()
+   * @model default="org.eclipse.emf.codegen.merge.java.facade.jdom.JDOMFacadeHelper"
+   * @generated
+   */
+  String getFacadeHelperClass();
+
+  /**
+   * Sets the value of the '{@link org.eclipse.emf.codegen.ecore.genmodel.GenModel#getFacadeHelperClass <em>Facade Helper Class</em>}' attribute.
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @param value the new value of the '<em>Facade Helper Class</em>' attribute.
+   * @see #getFacadeHelperClass()
+   * @generated
+   */
+  void setFacadeHelperClass(String value);
+
   /**
    * Returns the value of the '<em><b>Gen Packages</b></em>' containment reference list.
    * The list contents are of type {@link org.eclipse.emf.codegen.ecore.genmodel.GenPackage}.
@@ -1308,40 +1618,177 @@ public interface GenModel extends GenBase{
   void emitSortedImports();
   String getIndentation(StringBuffer stringBuffer);
 
+  ImportManager getImportManager();
+  void setImportManager(ImportManager importManager);
+
   String getImportedName(String qualifiedName);
   void addImport(String qualifiedName);
   void addPseudoImport(String qualifiedName);
 
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.Generator.Options Generator.Options} provides an
+   * equivalent way to obtain this information. This method will be removed after 2.2.
+   */
+  String getMergeRulesLocation();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.Generator Generator} provides an equivalent to this method.
+   * This method will be removed after 2.2.
+   */
   JControlModel getJControlModel();
 
-  JETEmitter getInterfaceEmitter();
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEnumClassEmitter();
-  JETEmitter getFactoryInterfaceEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getFactoryClassEmitter();
-  JETEmitter getPackageInterfaceEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getPackageClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getAdapterFactoryClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getSwitchClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getValidatorClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getPluginXMLEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getManifestMFEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getModelPluginClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getResourceClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getResourceFactoryClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getBuildPropertiesEmitter();
 
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
+  JETEmitter getXMLProcessorClassEmitter();
+
   //
   // EMF.Edit generation
   //
   boolean hasEditSupport();
 
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getItemProviderEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getItemProviderAdapterFactoryEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditPluginClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditPluginXMLEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditManifestMFEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditPluginPropertiesEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditBuildPropertiesEmitter();
 
   //
@@ -1349,14 +1796,67 @@ public interface GenModel extends GenBase{
   //
   boolean hasEditorSupport();
 
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getActionBarContributorEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getModelWizardEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorAdvisorEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorPluginClassEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorPluginXMLEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorManifestMFEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorPluginPropertiesEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getEditorBuildPropertiesEmitter();
 
   //
@@ -1364,15 +1864,62 @@ public interface GenModel extends GenBase{
   //
   boolean hasTestSupport();
 
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getTestCaseEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getModelTestSuiteEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getPackageTestSuiteEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getPackageExampleEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getTestsPluginXMLEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getTestsManifestMFEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getTestsPluginPropertiesEmitter();
+
+  /**
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
+   */
   JETEmitter getTestsBuildPropertiesEmitter();
-  
+
   // boolean isGenerateEditPlugin();
   // boolean isGenerateEditorPlugin();
 
@@ -1405,16 +1952,19 @@ public interface GenModel extends GenBase{
   String getModelPluginPackageName();
   String getQualifiedModelPluginClassName();
 
+  String getEditPluginDirectory();
   String getEditPluginClassName();
   String getEditPluginPackageName();
   String getQualifiedEditPluginClassName();
 
+  String getEditorPluginDirectory();
   String getEditorPluginClassName();
   String getEditorPluginPackageName();
   String getEditorAdvisorClassName();
   String getQualifiedEditorPluginClassName();
   String getQualifiedEditorAdvisorClassName();
 
+  boolean hasTestSuiteClass();
   String getTestSuiteClassName();
   String getTestSuitePackageName();
   String getQualifiedTestSuiteClassName();
@@ -1444,6 +1994,7 @@ public interface GenModel extends GenBase{
   boolean hasXMLDependency();
 
   IStatus validate();
+  Diagnostic diagnose();
 
   String getXMLEncodingChoices();
 
@@ -1455,12 +2006,19 @@ public interface GenModel extends GenBase{
   List/*of GenFeature*/ getFilteredAllGenFeatures(); // Filtered for property keys to remove duplicates.
 
   /**
-   * Set the code formatter options to be used to {@link #createCodeFormatter create} a new code formatter. 
+   * Set the code formatter options to be used to {@link #createCodeFormatter create} a new code formatter.
+   *  
+   * @deprecated In EMF 2.2, the {@link org.eclipse.emf.codegen.ecore.generator.Generator.Options Generator.Options} should be used to
+   * record code formatter options in order to be used via the new Generator-based design. This method will be removed after 2.2.
    */
   void setCodeFormatterOptions(Map options);
 
   /**
    * Creates and returns a new JDT code formatter.
+   * 
+   * @deprecated In EMF 2.2, a {@link org.eclipse.emf.codegen.ecore.generator.GeneratorAdapter GeneratorAdapter} should be used to
+   * implement code generation. {@link org.eclipse.emf.codegen.ecore.generator.AbstractGeneratorAdapter AbstractGeneratorAdapter} provides
+   * an equivalent to this method. This method will be removed after 2.2.
    */
   CodeFormatter createCodeFormatter();
 
@@ -1475,6 +2033,8 @@ public interface GenModel extends GenBase{
   GenDataType createGenDataType();
   GenOperation createGenOperation();
   GenParameter createGenParameter();
+  GenAnnotation createGenAnnotation();
+  GenBase create(EClass eClass);
 
   Set getPropertyCategories();
   
@@ -1486,4 +2046,6 @@ public interface GenModel extends GenBase{
   ExtendedMetaData getExtendedMetaData();
   
   List computeMissingUsedGenPackages();
+
+  boolean isVirtualDelegation();
 }
