@@ -1,7 +1,7 @@
 /**
  * <copyright> 
  *
- * Copyright (c) 2002-2005 IBM Corporation and others.
+ * Copyright (c) 2002-2006 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: EMFPlugin.java,v 1.10 2005/06/15 12:50:35 emerks Exp $
+ * $Id: EMFPlugin.java,v 1.14 2006/05/12 16:38:23 marcelop Exp $
  */
 package org.eclipse.emf.common;
 
@@ -27,7 +27,11 @@ import java.util.Map;
 import java.util.MissingResourceException;
 import java.util.PropertyResourceBundle;
 import java.util.ResourceBundle;
+import java.util.jar.Manifest;
 
+import org.osgi.framework.Bundle;
+
+import org.eclipse.core.runtime.ILog;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Platform;
 import org.eclipse.core.runtime.Plugin;
@@ -102,13 +106,15 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
 
   public String getSymbolicName()
   {
-    if (getPluginResourceLocator() instanceof EclipsePlugin)
+    ResourceLocator resourceLocator = getPluginResourceLocator();
+    if (resourceLocator instanceof InternalEclipsePlugin)
     {
-      return ((EclipsePlugin)getPluginResourceLocator()).getSymbolicName();
+      return ((InternalEclipsePlugin)resourceLocator).getSymbolicName();
     }
     else
     {
-      throw new UnsupportedOperationException("Plugin ID not available " + this);
+      String result = getClass().getName();
+      return result.substring(0, result.lastIndexOf('.'));
     }
   }
 
@@ -153,7 +159,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
             URL classURL = theClass.getResource((index == -1 ? className : className.substring(index + 1)) + ".class");
             URI uri = URI.createURI(classURL.toString());
             
-            // Trim off the segements corresponding to the package nesting.
+            // Trim off the segments corresponding to the package nesting.
             //
             int count = 1;
             for (int i = 0; (i = className.indexOf('.', i)) != -1; ++i)
@@ -188,7 +194,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
             //
             if (baseURL == null)
             {
-              // Trim off the "bin" or "runtime" segement.
+              // Trim off the "bin" or "runtime" segment.
               //
               String lastSegment = uri.lastSegment();
               if ("bin".equals(lastSegment) || "runtime".equals(lastSegment))
@@ -500,10 +506,13 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
   /**
    * The actual implementation of an Eclipse <b>Plugin</b>.
    */
-  public static abstract class EclipsePlugin extends Plugin implements ResourceLocator, Logger
+  public static abstract class EclipsePlugin extends Plugin implements ResourceLocator, Logger, InternalEclipsePlugin
   {
-    protected ResourceBundle resourceBundle;
-    protected ResourceBundle untranslatedResourceBundle;
+    /**
+     * The EMF plug-in APIs are all delegated to this helper, so that code can be shared by plug-in
+     * implementations with a different platform base class (e.g. AbstractUIPlugin).
+     */
+    protected InternalHelper helper;
     
     /**
      * Creates an instance.
@@ -511,6 +520,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
     public EclipsePlugin()
     {
       super();
+      helper = new InternalHelper(this);
     }
 
     /**
@@ -521,6 +531,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
     public EclipsePlugin(org.eclipse.core.runtime.IPluginDescriptor descriptor)
     {
       super(descriptor);
+      helper = new InternalHelper(this);
     }
 
     /**
@@ -528,7 +539,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
      */
     public String getSymbolicName()
     {
-      return getBundle().getSymbolicName();
+      return helper.getSymbolicName();
     }
 
     /*
@@ -536,7 +547,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
      */
     public URL getBaseURL()
     {
-      return getBundle().getEntry("/");
+      return helper.getBaseURL();
     }
 
     /*
@@ -570,6 +581,90 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
      * @return the description of the image associated with the key.
      */
     protected Object doGetImage(String key) throws IOException
+    {
+      return helper.getImage(key);
+    }
+
+    public String getString(String key)
+    {
+      return helper.getString(key, true);
+    }
+    
+    public String getString(String key, boolean translate)
+    {
+      return helper.getString(key, translate);
+    }
+
+    public String getString(String key, Object [] substitutions)
+    {
+      return helper.getString(key, substitutions, true);
+    }
+
+    public String getString(String key, Object [] substitutions, boolean translate)
+    {
+      return helper.getString(key, substitutions, translate);
+    }
+
+    public void log(Object logEntry)
+    {
+      helper.log(logEntry);
+    }
+  }
+
+  /**
+   * This just provides a common interface for the Eclipse plugins supported by EMF.
+   * It is not considered API and should not be used by clients.
+   */
+  public static interface InternalEclipsePlugin
+  {
+    String getSymbolicName();
+  }
+  
+  /**
+   * This just provides a common delegate for non-UI and UI plug-in classes.
+   * It is not considered API and should not be used by clients.
+   */
+  public static class InternalHelper
+  {
+    protected Plugin plugin;
+    protected ResourceBundle resourceBundle;
+    protected ResourceBundle untranslatedResourceBundle;
+
+    public InternalHelper(Plugin plugin)
+    {
+      this.plugin = plugin;
+    }
+
+    protected Bundle getBundle()
+    {
+      return plugin.getBundle();
+    }
+
+    protected ILog getLog()
+    {
+      return plugin.getLog();
+    }
+    
+    /**
+     * Return the plugin ID.
+     */
+    public String getSymbolicName()
+    {
+      return getBundle().getSymbolicName();
+    }
+
+    public URL getBaseURL()
+    {
+      return getBundle().getEntry("/");
+    }
+
+    /**
+     * Fetches the image associated with the given key. It ensures that the image exists.
+     * @param key the key of the image to fetch.
+     * @exception IOException if an image doesn't exist.
+     * @return the description of the image associated with the key.
+     */
+    public Object getImage(String key) throws IOException
     {
       URL url = new URL(getBaseURL() + "icons/" + key + ".gif");
       InputStream inputStream = url.openStream(); 
@@ -577,17 +672,6 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
       return url;
     }
 
-    /*
-     * Javadoc copied from interface.
-     */
-    public String getString(String key)
-    {
-      return getString(key, true);
-    }
-    
-    /*
-     * Javadoc copied from interface.
-     */
     public String getString(String key, boolean translate)
     {
       ResourceBundle bundle = translate ? resourceBundle : untranslatedResourceBundle;
@@ -615,25 +699,11 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
       return bundle.getString(key);
     }
 
-    /*
-     * Javadoc copied from interface.
-     */
-    public String getString(String key, Object [] substitutions)
-    {
-      return getString(key, substitutions, true);
-    }
-
-    /*
-     * Javadoc copied from interface.
-     */
     public String getString(String key, Object [] substitutions, boolean translate)
     {
       return MessageFormat.format(getString(key, translate), substitutions);
     }
 
-    /*
-     * Javadoc copied from interface.
-     */
     public void log(Object logEntry)
     {
       IStatus status;
@@ -646,7 +716,7 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
       {
         if (logEntry == null)
         {
-          logEntry = new RuntimeException(getString("_UI_NullLogEntry_exception")).fillInStackTrace();
+          logEntry = new RuntimeException(getString("_UI_NullLogEntry_exception", true)).fillInStackTrace();
         }
 
         if (logEntry instanceof Throwable)
@@ -672,6 +742,90 @@ public abstract class EMFPlugin implements ResourceLocator, Logger
           getLog().log (new Status (IStatus.WARNING, getBundle().getSymbolicName(), 0, logEntry.toString(), null));
         }
       }
+    } 
+  }
+
+  public static void main(String[] args)
+  {
+    try
+    {
+      String [] relativePath = { "META-INF", "MANIFEST.MF" };
+      Class theClass =  args.length > 0 ? Class.forName(args[0]) : EMFPlugin.class;
+
+      String className = theClass.getName();
+      int index = className.lastIndexOf(".");
+      URL classURL = theClass.getResource((index == -1 ? className : className.substring(index + 1)) + ".class");
+      URI uri = URI.createURI(classURL.toString());
+
+      // Trim off the segments corresponding to the package nesting.
+      //
+      int count = 1;
+      for (int i = 0; (i = className.indexOf('.', i)) != -1; ++i)
+      {
+        ++count;
+      }
+      uri = uri.trimSegments(count);
+
+      URL manifestURL = null;
+  
+      // For an archive URI, check for the path in the archive.
+      //
+      if (URI.isArchiveScheme(uri.scheme()))
+      {
+        try
+        {
+          // If we can open  an input stream, then the path is there, and we have a good URL.
+          //
+          String manifestURI = uri.appendSegments(relativePath).toString();
+          InputStream inputStream =  new URL(manifestURI).openStream();
+          inputStream.close();
+          manifestURL = new URL(manifestURI);
+        }
+        catch (IOException exception)
+        {
+          // If the path isn't within the root of the archive, 
+          // create a new URI for the folder location of the archive, 
+          // so we can look in the folder that contains it.
+          //
+          uri = URI.createURI(uri.authority()).trimSegments(1);
+        }
+      }
+              
+      // If we didn't find the path in the usual place nor in the archive...
+      //
+      if (manifestURL == null)
+      {
+        // Trim off the "bin" or "runtime" segment.
+        //
+        String lastSegment = uri.lastSegment();
+        if ("bin".equals(lastSegment) || "runtime".equals(lastSegment))
+        {
+          uri = uri.trimSegments(1);
+        }
+        uri = uri.appendSegments(relativePath);
+        manifestURL = new URL(uri.toString());
+      }
+              
+      if (manifestURL != null)
+      {
+        Manifest manifest = new Manifest(manifestURL.openStream());
+        String symbolicName =  manifest.getMainAttributes().getValue("Bundle-SymbolicName");
+        if (symbolicName != null)
+        {
+          int end = symbolicName.indexOf(";");
+          if (end != -1)
+          {
+            symbolicName = symbolicName.substring(0, end);
+          }
+          System.out.println("Bundle-SymbolicName=" + symbolicName + " Bundle-Version=" + manifest.getMainAttributes().getValue("Bundle-Version"));
+          return;
+        }
+      }
     }
+    catch (Exception exception)
+    {
+    }
+    
+    System.err.println("No Bundle information found");
   }
 }
