@@ -12,11 +12,14 @@
  *
  * </copyright>
  *
- * $Id: URIMappingRegistryReader.java,v 1.3 2005/06/08 06:20:10 nickb Exp $
+ * $Id: URIMappingRegistryReader.java,v 1.6 2006/05/25 19:17:08 emerks Exp $
  */
 package org.eclipse.emf.ecore.plugin;
 
 
+import java.util.HashMap;
+import java.util.Map;
+
 import org.eclipse.core.runtime.IConfigurationElement;
 import org.eclipse.core.runtime.Platform;
 
@@ -35,6 +38,8 @@ class URIMappingRegistryReader extends RegistryReader
   static final String ATT_SOURCE   = "source";
   static final String ATT_TARGET   = "target";
    
+  protected Map map = new HashMap();
+
   public URIMappingRegistryReader() 
   {
     super
@@ -68,9 +73,15 @@ class URIMappingRegistryReader extends RegistryReader
             targetURI = 
               targetURI.resolve
                 (URI.createURI
-                  (Platform.getBundle(element.getDeclaringExtension().getNamespace()).getEntry("/").toString()));
+                  (Platform.getBundle(element.getDeclaringExtension().getContributor().getName()).getEntry("/").toString()));
           }
           URIConverter.URI_MAP.put(sourceURI, targetURI);
+          IConfigurationElement previous = (IConfigurationElement)map.put(sourceURI, element);
+          if (previous != null)
+          {
+            EcorePlugin.INSTANCE.log
+              ("Both '" + previous.getContributor().getName() + "' and '" + element.getContributor().getName() + "' register a URI mapping for '" + sourceURI + "'");
+          }
           return true;
         }
       }
