@@ -1,7 +1,7 @@
 /**
  * <copyright>
  *
- * Copyright (c) 2002-2004 IBM Corporation and others.
+ * Copyright (c) 2002-2005 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: ResourceImpl.java,v 1.9 2005/06/08 06:20:10 nickb Exp $
+ * $Id: ResourceImpl.java,v 1.17 2006/05/03 16:50:49 emerks Exp $
  */
 package org.eclipse.emf.ecore.resource.impl;
 
@@ -50,6 +50,8 @@ import org.eclipse.emf.ecore.resource.ResourceSet;
 import org.eclipse.emf.ecore.resource.URIConverter;
 import org.eclipse.emf.ecore.util.EcoreUtil;
 import org.eclipse.emf.ecore.util.InternalEList;
+import org.eclipse.emf.ecore.util.EcoreUtil.ContentTreeIterator;
+import org.eclipse.emf.ecore.util.EcoreUtil.ProperContentIterator;
 
 
 /**
@@ -111,6 +113,29 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     }
     return defaultURIConverter;
   }
+  
+  /**
+   * Merges 2 maps, without changing any of them.  If map2 and map1
+   * have the same key for an entry, map1's value will be the one in 
+   * the merged map.
+   */
+  protected static Map mergeMaps(Map map1, Map map2)
+  {
+    if (map1 == null || map1.isEmpty())
+    {
+      return map2;
+    }
+    else if (map2 == null || map2.isEmpty())
+    {
+      return map1;
+    }
+    else
+    {
+      Map mergedMap = new HashMap(map2);
+      mergedMap.putAll(map1);
+      return mergedMap;
+    }
+  }
 
   /**
    * The storage for the default save options.
@@ -164,6 +189,12 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
    */
   protected boolean isLoaded;
 
+  /**
+   * The loading flag.
+   * @see #isLoading
+   */
+  protected boolean isLoading;
+  
   /**
    * The modification tracking adapter.
    * @see #isTrackingModification
@@ -356,7 +387,10 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     protected void didAdd(int index, Object object)
     {
       super.didAdd(index, object);
-      loaded();
+      if (index == size - 1)
+      {
+        loaded();
+      }
       modified();
     }
 
@@ -431,6 +465,23 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
         }
       };
   }
+  
+  protected TreeIterator getAllProperContents(EObject eObject)
+  {
+    return EcoreUtil.getAllProperContents(eObject, false);
+  }
+
+  protected TreeIterator getAllProperContents(List contents)
+  {
+    return  
+      new ContentTreeIterator(contents, false)
+      {
+        public Iterator getChildren(Object object)
+        {
+          return object == this.object ? ((List)object).iterator() : new ProperContentIterator(((EObject)object));
+        }
+      };
+  }
 
   /*
    * Javadoc copied from interface.
@@ -517,7 +568,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
   {
     List contents = getContents();
     return contents.size() > 1 ?
-      Integer.toString(getContents().indexOf(eObject)) :
+      Integer.toString(contents.indexOf(eObject)) :
       "";
   }
 
@@ -533,22 +584,34 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     }
     else
     {
-      List uriFragmentPath = new ArrayList();
-      for (EObject container = eObject.eContainer(); container != null; container = eObject.eContainer())
+      InternalEObject internalEObject = (InternalEObject)eObject;
+      if (internalEObject.eDirectResource() == this)
       {
-        uriFragmentPath.add(((InternalEObject)container).eURIFragmentSegment(eObject.eContainingFeature(), eObject));
-        eObject = container;
+        return "/" + getURIFragmentRootSegment(eObject);
       }
-
-      StringBuffer result = new StringBuffer("/");
-      result.append(getURIFragmentRootSegment(eObject));
-
-      for (ListIterator i = uriFragmentPath.listIterator(uriFragmentPath.size()); i.hasPrevious(); )
+      else
       {
-        result.append('/');
-        result.append((String)i.previous());
+        List uriFragmentPath = new ArrayList();
+        for (InternalEObject container = internalEObject.eInternalContainer(); container != null; container = internalEObject.eInternalContainer())
+        {
+          uriFragmentPath.add(container.eURIFragmentSegment(internalEObject.eContainingFeature(), internalEObject));
+          internalEObject = container;
+          if (container.eDirectResource() == this)
+          {
+            break;
+          }
+        }
+  
+        StringBuffer result = new StringBuffer("/");
+        result.append(getURIFragmentRootSegment(internalEObject));
+  
+        for (int i = uriFragmentPath.size() - 1; i >= 0; --i)
+        {
+          result.append('/');
+          result.append(uriFragmentPath.get(i));
+        }
+        return result.toString();
       }
-      return result.toString();
     }
   }
 
@@ -676,7 +739,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
       }
     }
     
-    for (Iterator i = getAllContents(); i.hasNext(); )
+    for (TreeIterator i = getAllProperContents(getContents()); i.hasNext(); )
     {
       EObject eObject = (EObject)i.next();
       String eObjectId = EcoreUtil.getID(eObject);
@@ -686,7 +749,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
         {
           map.put(eObjectId, eObject);
         }
-        
+          
         if (eObjectId.equals(id))
         {
           return eObject;
@@ -702,7 +765,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     if (isAttachedDetachedHelperRequired())
     {
       attachedHelper(eObject);
-      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
+      for (TreeIterator tree = getAllProperContents(eObject); tree.hasNext(); )
       {
         attachedHelper((EObject)tree.next());
       }
@@ -749,7 +812,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     if (isAttachedDetachedHelperRequired())
     {
       detachedHelper(eObject);
-      for (Iterator tree = eObject.eAllContents(); tree.hasNext(); )
+      for (TreeIterator tree = getAllProperContents(eObject); tree.hasNext(); )
       {
         detachedHelper((EObject)tree.next());
       }
@@ -863,15 +926,49 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
    */
   public final void save(OutputStream outputStream, Map options) throws IOException
   {
+    if (errors != null)
+    {
+      errors.clear();
+    }
+
+    if (warnings != null)
+    {
+      warnings.clear();
+    }
+
+    options = mergeMaps(options, defaultSaveOptions);
+    URIConverter.Cipher cipher = options != null ? 
+      (URIConverter.Cipher)options.get(Resource.OPTION_CIPHER) : 
+      null;
+      
+    if (cipher != null)
+    {
+      try
+      {
+        outputStream = cipher.encrypt(outputStream);
+      }
+      catch (Exception e)
+      {
+        throw new IOWrappedException(e);
+      }
+    }
+
     ZipOutputStream zipOutputStream = null;
-    if (useZip())
+    if (useZip() || (options != null && Boolean.TRUE.equals(options.get(Resource.OPTION_ZIP))))
     {
       zipOutputStream = 
         new ZipOutputStream(outputStream)
         {
+          public void finish() throws IOException
+          {
+            super.finish();
+            def.end();
+          }
+
           public void flush()
           {
           }
+
           public void close() throws IOException
           {
             try
@@ -887,24 +984,23 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
       zipOutputStream.putNextEntry(newContentZipEntry());
       outputStream = zipOutputStream;
     }
+        
+    doSave(outputStream, options);
 
-    if (defaultSaveOptions == null || defaultSaveOptions.isEmpty())
+    if (cipher != null)
     {
-      doSave(outputStream, options);
+      try
+      {
+        cipher.finish(outputStream);
+      }
+      catch (Exception e)
+      {
+        throw new IOWrappedException(e);
+      }
     }
-    else if (options == null)
-    {
-      doSave(outputStream, defaultSaveOptions);
-    }
-    else
-    {
-      Map mergedOptions = new HashMap(defaultSaveOptions);
-      mergedOptions.putAll(options);
-
-      doSave(outputStream, mergedOptions);
-    }
-
+    
     setModified(false);
+
     if (zipOutputStream != null)
     {
       zipOutputStream.finish();
@@ -945,6 +1041,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     if (!isLoaded)
     {
       Notification notification = setLoaded(true);
+      isLoading = true;
 
       if (errors != null)
       {
@@ -956,40 +1053,57 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
         warnings.clear();
       }
 
-      if (useZip())
-      {
-        ZipInputStream zipInputStream = new ZipInputStream(inputStream);
-        while (zipInputStream.available() != 0)
-        {
-          ZipEntry zipEntry = zipInputStream.getNextEntry();
-          if (isContentZipEntry(zipEntry))
-          {
-            inputStream = zipInputStream;
-            break;
-          }
-        }
-      }
-
       try
       {
-        if (defaultLoadOptions == null || defaultLoadOptions.isEmpty())
+        if (useZip() || (options != null && Boolean.TRUE.equals(options.get(Resource.OPTION_ZIP))))
         {
-          doLoad(inputStream, options);
+          ZipInputStream zipInputStream = new ZipInputStream(inputStream);
+          while (zipInputStream.available() != 0)
+          {
+            ZipEntry zipEntry = zipInputStream.getNextEntry();
+            if (isContentZipEntry(zipEntry))
+            {
+              inputStream = zipInputStream;
+              break;
+            }
+          }
         }
-        else if (options == null)
+
+        options = mergeMaps(options, defaultLoadOptions);
+        URIConverter.Cipher cipher = options != null ?
+          (URIConverter.Cipher)options.get(Resource.OPTION_CIPHER) :
+          null;
+          
+        if (cipher != null)
         {
-          doLoad(inputStream, defaultLoadOptions);
+          try
+          {
+            inputStream = cipher.decrypt(inputStream);
+          }
+          catch (Exception e)
+          {
+            throw new IOWrappedException(e);
+          }
         }
-        else
+        
+        doLoad(inputStream, options);
+        
+        if (cipher != null)
         {
-          Map mergedOptions = new HashMap(defaultLoadOptions);
-          mergedOptions.putAll(options);
-  
-          doLoad(inputStream, mergedOptions);
-        }
+          try
+          {
+            cipher.finish(inputStream);
+          }
+          catch (Exception e)
+          {
+            throw new IOWrappedException(e);
+          }
+        }        
       }
       finally
       {
+        isLoading = false;
+
         if (notification != null)
         {
           eNotify(notification);
@@ -1021,6 +1135,14 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
     return isLoaded;
   }
 
+  /*
+   * Javadoc copied from interface.
+   */
+  public boolean isLoading()
+  {
+    return isLoading;
+  }
+
   /**
    * Called when the object is unloaded.
    * This implementation 
@@ -1073,7 +1195,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
    */
   protected void doUnload()
   {
-    Iterator allContents = EcoreUtil.getAllContents(new ArrayList(getContents()));
+    Iterator allContents = getAllProperContents(new ArrayList(getContents()));
 
     // This guard is needed to ensure that clear doesn't make the resource become loaded.
     //
@@ -1158,7 +1280,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
       {
         modificationTrackingAdapter = createModificationTrackingAdapter();
         
-        for (Iterator i = getAllContents(); i.hasNext(); )
+        for (TreeIterator i = getAllProperContents(getContents()); i.hasNext(); )
         {
           EObject eObject = (EObject)i.next();
           eObject.eAdapters().add(modificationTrackingAdapter);
@@ -1169,7 +1291,7 @@ public class ResourceImpl extends NotifierImpl implements Resource, Resource.Int
         Adapter oldModificationTrackingAdapter = modificationTrackingAdapter;
         modificationTrackingAdapter = null;
         
-        for (Iterator i = getAllContents(); i.hasNext(); )
+        for (TreeIterator i = getAllProperContents(getContents()); i.hasNext(); )
         {
           EObject eObject = (EObject)i.next();
           eObject.eAdapters().remove(oldModificationTrackingAdapter);
