@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: XMLHelperImpl.java,v 1.29 2005/06/10 13:57:51 elena Exp $
+ * $Id: XMLHelperImpl.java,v 1.34 2006/05/12 15:50:44 emerks Exp $
  */
 package org.eclipse.emf.ecore.xmi.impl;
 
@@ -66,6 +66,7 @@ public class XMLHelperImpl implements XMLHelper
   protected static final Integer INTEGER_DATATYPE_SINGLE  = new Integer(DATATYPE_SINGLE);
   protected static final Integer INTEGER_IS_MANY_ADD      = new Integer(IS_MANY_ADD);
   protected static final Integer INTEGER_IS_MANY_MOVE     = new Integer(IS_MANY_MOVE);
+  protected static final Integer INTEGER_OTHER     = new Integer(OTHER);
 
   protected EPackage noNamespacePackage;
   protected XMLResource.XMLMap xmlMap;
@@ -80,6 +81,7 @@ public class XMLHelperImpl implements XMLHelper
   protected String processDanglingHREF;
   protected DanglingHREFException danglingHREFException;
   protected EMap prefixesToURIs;
+  protected Map urisToPrefixes;
   protected Map anyPrefixesToURIs;
   protected NamespaceSupport namespaceSupport;
   protected EClass anySimpleType;
@@ -130,7 +132,44 @@ public class XMLHelperImpl implements XMLHelper
     super();
     packages = new HashMap();
     featuresToKinds = new HashMap();
-    prefixesToURIs = new BasicEMap();
+    prefixesToURIs = 
+      new BasicEMap()
+      {
+        protected List getPrefixes(Object uri)
+        {
+          List result = (List)urisToPrefixes.get(uri);
+          if (result == null)
+          {
+            urisToPrefixes.put(uri, result = new ArrayList());
+          }
+          return result;
+        }
+
+        protected void didAdd(Entry entry)
+        {
+          getPrefixes(entry.getValue()).add(entry.getKey());
+        }
+
+        protected void didClear(BasicEList[] oldEntryData)
+        {
+          urisToPrefixes.clear();
+        }
+
+        protected void didModify(Entry entry, Object oldValue)
+        {
+          Object key = entry.getKey();
+          getPrefixes(oldValue).remove(key);
+          getPrefixes(entry.getValue()).add(key);
+        }
+
+        protected void didRemove(Entry entry)
+        {
+          getPrefixes(entry.getValue()).add(entry.getKey());
+        }
+      };
+
+    urisToPrefixes = new HashMap();
+      
     anyPrefixesToURIs = new HashMap();
     allPrefixToURI = new ArrayList();
     namespaceSupport = new NamespaceSupport();
@@ -472,12 +511,12 @@ public class XMLHelperImpl implements XMLHelper
             extendedMetaData.getNamespace(ePackage);
 
       boolean found = false;
-      for (Iterator i = prefixesToURIs.entrySet().iterator(); i.hasNext(); )
+      List prefixes = (List)urisToPrefixes.get(nsURI);
+      if (prefixes != null)
       {
-        Map.Entry entry = (Map.Entry)i.next();
-        if (nsURI == null ? entry.getValue() == null : nsURI.equals(entry.getValue()))
+        for (Iterator i = prefixes.iterator(); i.hasNext(); )
         {
-          nsPrefix = (String)entry.getKey();
+          nsPrefix = (String)i.next();
           if (!mustHavePrefix || nsPrefix.length() > 0)
           {
             found = true;
@@ -535,13 +574,10 @@ public class XMLHelperImpl implements XMLHelper
     List result = new UniqueEList();
     result.add(getPrefix(ePackage));
     String namespace = extendedMetaData == null ? ePackage.getNsURI() : extendedMetaData.getNamespace(ePackage);
-    for (Iterator i = prefixesToURIs.entrySet().iterator(); i.hasNext(); )
+    List prefixes = (List)urisToPrefixes.get(namespace);
+    if (prefixes != null)
     {
-      Map.Entry entry = (Map.Entry)i.next();
-      if (namespace == null ? entry.getValue() == null : namespace.equals(entry.getValue()))
-      {
-        result.add(entry.getKey());
-      }
+      result.addAll(prefixes);
     }
     return result;
   }
@@ -666,10 +702,17 @@ public class XMLHelperImpl implements XMLHelper
       Resource otherResource = obj.eResource();
       if (otherResource == null)
       {
-        objectURI = handleDanglingHREF(obj);
-        if (objectURI == null)
+        if (resource != null && resource.getID(obj) != null)
         {
-          return null;
+          objectURI = getHREF(resource, obj);
+        }
+        else
+        {
+          objectURI = handleDanglingHREF(obj);
+          if (objectURI == null)
+          {
+            return null;
+          }
         }
       }
       else
@@ -719,60 +762,101 @@ public class XMLHelperImpl implements XMLHelper
       }
       else
       {
+        featuresToKinds.put(feature, INTEGER_OTHER);
         return OTHER;
       }
     }
   }
+  
+  public EObject createObject(EFactory eFactory, EClassifier type)
+  {
+    EObject newObject = null;
+    if (eFactory != null)
+    {
+      if (extendedMetaData != null)
+      {
+        if (type == null)
+        {
+          return null;
+        }
+        else if (type instanceof EClass)
+        {
+          newObject = eFactory.create((EClass)type);
+        }
+        else
+        {
+          SimpleAnyType result = (SimpleAnyType)EcoreUtil.create(anySimpleType);
+          result.setInstanceType((EDataType)type);
+          newObject = result;
+        }
+      }
+      else
+      {
+        if (type != null)
+        {
+          newObject = eFactory.create((EClass)type);
+        }
+      }
+    }
+    return newObject;
+  }
+  
+  
+  public EClassifier getType(EFactory eFactory, String typeName)
+  {
+    if (eFactory != null)
+    {
+      EPackage ePackage = eFactory.getEPackage();
+      if (extendedMetaData != null)
+      {
+        return extendedMetaData.getType(ePackage, typeName);
+      }
+      else
+      {
+        EClass eClass = (EClass)ePackage.getEClassifier(typeName);
+        if (eClass == null && xmlMap != null)
+        {
+          return (EClass)xmlMap.getClassifier(ePackage.getNsURI(), typeName);
+        }
+        return eClass;
+      }
+    }
+    return null;
+  }
 
+  /**
+   * @deprecated since 2.2
+   */
   public EObject createObject(EFactory eFactory, String classXMIName)
   {
-    EPackage ePackage = eFactory.getEPackage();
-    if (extendedMetaData != null)
-    {
-      EClassifier eClassifier = extendedMetaData.getType(ePackage, classXMIName);
-      if (eClassifier == null)
-      {
-        return null;
-      }
-      else if (eClassifier instanceof EClass)
-      {
-        return eFactory.create((EClass)eClassifier);
-      }
-      else
-      {
-        SimpleAnyType result = (SimpleAnyType)EcoreUtil.create(anySimpleType);
-        result.setInstanceType((EDataType)eClassifier);
-        return result;
-      }
-    }
-    else
-    {
-      EClass eClass = (EClass)ePackage.getEClassifier(classXMIName);
-      if (eClass == null && xmlMap != null)
-      {
-        eClass = (EClass) xmlMap.getClassifier(ePackage.getNsURI(), classXMIName);
-      }
-  
-      if (eClass != null)
-      {
-        return eFactory.create(eClass);
-      }
-      else
-      {
-        return null;
-      }
-    }
+    return createObject(eFactory, getType(eFactory, classXMIName));
   }
 
   public EStructuralFeature getFeature(EClass eClass, String namespaceURI, String name)
   {
     EStructuralFeature feature = getFeatureWithoutMap(eClass, name);
-    if (feature == null && xmlMap != null)
+    if (feature == null)
     {
-      feature = xmlMap.getFeature(eClass, namespaceURI, name);
-      if (feature != null)
+      if (xmlMap != null)
       {
-        computeFeatureKind(feature);
+        feature = xmlMap.getFeature(eClass, namespaceURI, name);
+        if (feature != null)
+        {
+          computeFeatureKind(feature);
+        }
+      }
+      else if (laxFeatureProcessing && extendedMetaData != null)
+      {
+        List structuralFeatures = eClass.getEAllStructuralFeatures();
+        for (int i = 0, size = structuralFeatures.size(); i < size; ++i)
+        {
+          EStructuralFeature eStructuralFeature = (EStructuralFeature)structuralFeatures.get(i);
+          if (name.equals(extendedMetaData.getName(eStructuralFeature))
+            && (namespaceURI == null ? extendedMetaData.getNamespace(eStructuralFeature) == null : namespaceURI.equals(extendedMetaData.getNamespace(eStructuralFeature))))
+          {
+            return eStructuralFeature;
+          }
+        }
       }
     }
 
@@ -1048,11 +1132,20 @@ public class XMLHelperImpl implements XMLHelper
     {
       for (int i = 0, l = values.length; i < l; i++)
       {
-        if (values[i] != null)
+        Object value = values[i];
+        if (value != null)
         {
           try
           {
-            list.move(positions[i], values[i]);
+            int sourcePosition = list.indexOf(value);
+            if (sourcePosition != -1)
+            {
+              list.move(positions[i], sourcePosition);
+            }
+            else
+            {
+              list.addUnique(positions[i], value);
+            }
           }
           catch (RuntimeException e)
           {
@@ -1110,6 +1203,11 @@ public class XMLHelperImpl implements XMLHelper
     namespaceSupport.popContext();
   }
 
+  public void popContext(Map prefixesToFactories)
+  {
+    namespaceSupport.popContext(prefixesToFactories);
+  }
+
   public void addPrefix(String prefix, String uri) 
   {
     if (!"xml".equals(prefix) && !"xmlns".equals(prefix))
@@ -1121,6 +1219,11 @@ public class XMLHelperImpl implements XMLHelper
     }
   }
   
+  public String getPrefix (String namespaceURI)
+  {
+    return namespaceSupport.getPrefix(namespaceURI);
+  }
+  
   public Map getAnyContentPrefixToURIMapping()
   {
     anyPrefixesToURIs.clear();
@@ -1265,6 +1368,15 @@ public class XMLHelperImpl implements XMLHelper
       namespaceSize = context[currentContext--];
     } 
 
+    public void popContext(Map prefixesToFactories)
+    {
+      int oldNamespaceSize = namespaceSize;
+      for (int i = namespaceSize = context[currentContext--]; i < oldNamespaceSize; i += 2)
+      {
+        prefixesToFactories.remove(namespace[i]);
+      }
+    } 
+
     /**
      * @param prefix prefix to declare
      * @param uri uri that maps to the prefix
@@ -1318,9 +1430,11 @@ public class XMLHelperImpl implements XMLHelper
       // find uri in current context
       for (int i = namespaceSize; i > 0; i -= 2) 
       {
-        if (namespace[i - 1].equals(uri)) 
+        String knownURI = namespace[i - 1];
+        if ((knownURI != null)? knownURI.equals(uri) : uri == knownURI) 
         {
-          if (getURI(namespace[i - 2]).equals(uri))
+          knownURI = getURI(namespace[i - 2]);
+          if ((knownURI != null)? knownURI.equals(uri) : uri == knownURI)
             return namespace[i - 2];
         }
       }
@@ -1448,7 +1562,7 @@ public class XMLHelperImpl implements XMLHelper
 
     return (!list) ? factory.convertToString(dataType, value) : null;
   }
-  
+
   protected void addNSDeclaration(String prefix, String uri)
   {
     if (uri != null)
