@@ -1,7 +1,7 @@
 /**
  * <copyright> 
  *
- * Copyright (c) 2002-2005 IBM Corporation and others.
+ * Copyright (c) 2002-2006 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: ItemProviderAdapter.java,v 1.19 2005/06/12 13:32:37 emerks Exp $
+ * $Id: ItemProviderAdapter.java,v 1.25 2006/06/02 22:01:02 davidms Exp $
  */
 package org.eclipse.emf.edit.provider;
 
@@ -61,6 +61,7 @@ import org.eclipse.emf.edit.command.MoveCommand;
 import org.eclipse.emf.edit.command.RemoveCommand;
 import org.eclipse.emf.edit.command.ReplaceCommand;
 import org.eclipse.emf.edit.command.SetCommand;
+import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
 import org.eclipse.emf.edit.domain.EditingDomain;
 
 
@@ -568,6 +569,21 @@ public class ItemProviderAdapter
     return null;
   }
 
+  /**
+   * This adds an overlay to the given image if the object is controlled.
+   */
+  protected Object overlayImage(Object object, Object image)
+  {
+    if (AdapterFactoryEditingDomain.isControlled(object))
+    {
+      List images = new ArrayList(2);
+      images.add(image);
+      images.add(EMFEditPlugin.INSTANCE.getImage("full/ovr16/ControlledObject"));
+      image = new ComposedImage(images);
+    }
+    return image;
+  }
+
   /**
    * This implements {@link IItemLabelProvider#getText IItemLabelProvider.getText} by simply calling toString on the argument.
    * This will often be correct as is.
@@ -587,6 +603,26 @@ public class ItemProviderAdapter
     return getText(object);
   }
 
+  /**
+   * This crops the given text to exclude any control characters. The first such character and all following it are replaced by "..."
+   * @since 2.2.0
+   */
+  public String crop(String text)
+  {
+    if (text != null)
+    {
+      char[] chars = text.toCharArray();
+      for (int i = 0; i < chars.length; i++)
+      {
+        if (Character.isISOControl(chars[i]))
+        {
+          return text.substring(0, i) + "...";
+        }
+      }
+    }
+    return text;
+  }
+
   /**
    * This implements {@link IEditingDomainItemProvider#getNewChildDescriptors
    * IEditingDomainItemProvider.getNewChildDescriptors}, returning {@link
@@ -1387,6 +1423,22 @@ public class ItemProviderAdapter
     {
       targets.remove(target);
     }
+
+    if (childrenStoreMap != null)
+    {
+      ChildrenStore store = (ChildrenStore)childrenStoreMap.remove(target);
+      if (store != null && wrappers != null)
+      {
+        for (Iterator i = store.getChildren().iterator(); i.hasNext(); )
+        {
+          Object child = i.next();
+          if (wrappers.remove(child))
+          {
+            ((IDisposable)child).dispose();
+          }
+        }
+      }
+    }
   }
 
   /**
@@ -1545,7 +1597,7 @@ public class ItemProviderAdapter
       child = entry.getValue();        
     }
 
-    if (feature instanceof EReference) //EStructuralFeature)
+    if (feature instanceof EReference)
     {
       EStructuralFeature eFeature = (EStructuralFeature)feature;
       String name = "full/ctool16/Create" + eFeature.getEContainingClass().getName() + "_" + eFeature.getName();
@@ -1561,7 +1613,21 @@ public class ItemProviderAdapter
       }
       catch (Exception e)
       {
-        System.out.println(name);
+        List images = new ArrayList();
+        IItemLabelProvider itemLabelProvider = 
+          (IItemLabelProvider)((ComposeableAdapterFactory)adapterFactory).getRootAdapterFactory().adapt(child, IItemLabelProvider.class);
+        images.add(itemLabelProvider.getImage(child));
+        images.add(EMFEditPlugin.INSTANCE.getImage("full/ovr16/CreateChild"));
+        return 
+          new ComposedImage(images)
+          {
+            public List getDrawPoints(Size size)
+            {
+              List result = super.getDrawPoints(size);
+              ((Point)result.get(1)).x = size.width - 7;
+              return result;
+            }
+          };
       }
     }
 
@@ -2579,6 +2645,10 @@ public class ItemProviderAdapter
   /**
    * This creates an item provider descriptor that uses a resource locator; specifies a static image, a category, and filter flags;
    * and determines the cell editor from the type of the structural feature. 
+   * <p>
+   * Newly regenerated code will no longer use this method. You'll need to override {@link
+   * #createItemPropertyDescriptor(AdapterFactory, ResourceLocator, String, String, EStructuralFeature, boolean, boolean, Object, String, String[])
+   * this form} instead.
    */
   protected ItemPropertyDescriptor createItemPropertyDescriptor(
     AdapterFactory adapterFactory,
@@ -2590,6 +2660,38 @@ public class ItemProviderAdapter
     Object staticImage,
     String category,
     String[] filterFlags)
+  {
+    return createItemPropertyDescriptor(
+      adapterFactory,
+      resourceLocator,
+      displayName,
+      description,
+      feature,
+      isSettable,
+      false,
+      false,
+      staticImage,
+      category,
+      filterFlags);
+  }
+
+  /**
+   * Creates an instance that uses a resource locator; indicates whether to be mutli-line and to sort choices; specifies
+   * a  static image, a category, and filter flags; and determines the cell editor from the type of the structural feature. 
+   * @since 2.2.0
+   */
+  protected ItemPropertyDescriptor createItemPropertyDescriptor(
+    AdapterFactory adapterFactory,
+    ResourceLocator resourceLocator,
+    String displayName,
+    String description,
+    EStructuralFeature feature,
+    boolean isSettable,
+    boolean multiLine,
+    boolean sortChoices,
+    Object staticImage,
+    String category,
+    String[] filterFlags)
   {
     return new ItemPropertyDescriptor(
       adapterFactory,
@@ -2598,6 +2700,8 @@ public class ItemProviderAdapter
       description,
       feature,
       isSettable,
+      multiLine,
+      sortChoices,
       staticImage,
       category,
       filterFlags);
