@@ -12,11 +12,10 @@
  *
  * </copyright>
  *
- * $Id: ModelImporter.java,v 1.18 2005/06/28 23:55:17 marcelop Exp $
+ * $Id: ModelImporter.java,v 1.27 2006/05/26 20:06:44 marcelop Exp $
  */
 package org.eclipse.emf.importer;
 
-import java.io.File;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -30,29 +29,30 @@ import java.util.StringTokenizer;
 
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
-import org.eclipse.core.resources.IResource;
-import org.eclipse.core.resources.IWorkspace;
 import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
 import org.eclipse.core.runtime.IPath;
-import org.eclipse.core.runtime.IProgressMonitor;
-import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.Platform;
-import org.eclipse.core.runtime.Status;
 
 import org.eclipse.emf.codegen.ecore.Generator;
 import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
 import org.eclipse.emf.codegen.ecore.genmodel.GenModelFactory;
+import org.eclipse.emf.codegen.ecore.genmodel.GenModelPackage;
 import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
 import org.eclipse.emf.codegen.util.CodeGenUtil;
 import org.eclipse.emf.common.CommonPlugin;
 import org.eclipse.emf.common.util.AbstractTreeIterator;
+import org.eclipse.emf.common.util.BasicDiagnostic;
+import org.eclipse.emf.common.util.Diagnostic;
+import org.eclipse.emf.common.util.DiagnosticException;
+import org.eclipse.emf.common.util.Monitor;
 import org.eclipse.emf.common.util.TreeIterator;
 import org.eclipse.emf.common.util.URI;
 import org.eclipse.emf.common.util.UniqueEList;
+import org.eclipse.emf.converter.ConverterPlugin;
+import org.eclipse.emf.converter.ModelConverter;
+import org.eclipse.emf.converter.util.ConverterUtil;
 import org.eclipse.emf.ecore.EPackage;
-import org.eclipse.emf.ecore.plugin.EcorePlugin;
 import org.eclipse.emf.ecore.resource.Resource;
 import org.eclipse.emf.ecore.resource.ResourceSet;
 import org.eclipse.emf.importer.util.ImporterUtil;
@@ -61,58 +61,12 @@ import org.eclipse.emf.importer.util.ImporterUtil;
 /**
  * @since 2.1.0
  */
-public abstract class ModelImporter
+public abstract class ModelImporter extends ModelConverter
 {
-  public static class EPackageList extends UniqueEList.FastCompare
+  public static final String GENANNOTATION_SOURCE_PREFIX = GenModelPackage.eNS_URI + "/importer/";
+  
+  public static class EPackageImportInfo extends ModelConverter.EPackageConvertInfo
   {
-    public EPackageList()
-    {
-      super();
-    }
-
-    public EPackageList(Collection collection)
-    {
-      super(collection);
-    }
-
-    public EPackageList(int initialCapacity)
-    {
-      super(initialCapacity);
-    }
-
-    protected Object[] newData(int capacity)
-    {
-      return new EPackage [capacity];
-    }
-  }
-
-  public static class GenPackageList extends UniqueEList.FastCompare
-  {
-    public GenPackageList()
-    {
-      super();
-    }
-
-    public GenPackageList(Collection collection)
-    {
-      super(collection);
-    }
-
-    public GenPackageList(int initialCapacity)
-    {
-      super(initialCapacity);
-    }
-
-    protected Object[] newData(int capacity)
-    {
-      return new GenPackage [capacity];
-    }
-  }
-
-  public static class EPackageInfo
-  {
-    protected boolean generate = false;
-    protected String ecoreFileName;
     protected String basePackage;
     protected String prefix;
 
@@ -128,22 +82,12 @@ public abstract class ModelImporter
 
     public String getEcoreFileName()
     {
-      return ecoreFileName;
+      return getConvertData();
     }
 
     public void setEcoreFileName(String ecoreFileName)
     {
-      this.ecoreFileName = ecoreFileName;
-    }
-
-    public boolean isGenerate()
-    {
-      return generate;
-    }
-
-    public void setGenerate(boolean generate)
-    {
-      this.generate = generate;
+      setConvertData(ecoreFileName);
     }
 
     public String getPrefix()
@@ -157,22 +101,6 @@ public abstract class ModelImporter
     }
   }
   
-  protected static class ShellFinder
-  {
-    public Object getActiveShell()
-    {
-      Object shell = null;
-      try
-      {
-        shell = org.eclipse.swt.widgets.Display.getCurrent().getActiveShell();
-      }
-      catch (Throwable t)
-      {
-      }
-      return shell;
-    }
-  }
-
   protected List fileExtensions;
 
   protected IPath originalGenModelPath;
@@ -182,12 +110,6 @@ public abstract class ModelImporter
   protected IPath genModelContainerPath;
   protected String genModelFileName;
   protected IPath genModelPath;
-  protected GenModel genModel;
-
-  protected List ePackages;
-  protected Map ePackageToInfoMap;
-
-  protected List referencedGenPackages;
 
   protected List modelLocationURIs;
   protected String modelLocation;
@@ -198,25 +120,19 @@ public abstract class ModelImporter
   protected boolean usePlatformURI = true;
   protected IWorkspaceRoot workspaceRoot;
   
-  protected ResourceSet externalGenModelResourceSet;
-  protected List externalGenModelList;
-
   public void dispose()
   {
-    genModel = null;
     originalGenModel = null;
     workspaceRoot = null;
 
-    if (referencedGenPackages != null)
-    {
-      referencedGenPackages.clear();
-      referencedGenPackages = null;
-    }
-    clearEPackagesCollections();
+    super.dispose();
   }
-
-  public abstract String getID();
-
+  
+  protected String getConverterGenAnnotationSource()
+  {
+    return GENANNOTATION_SOURCE_PREFIX + getID();
+  }
+  
   public List getFileExtensions()
   {
     if (fileExtensions == null)
@@ -236,7 +152,7 @@ public abstract class ModelImporter
     this.usePlatformURI = usePlatformURI;
   }
 
-  public void defineOriginalGenModelPath(IPath path)
+  public void defineOriginalGenModelPath(IPath path) throws DiagnosticException
   {
     if (getOriginalGenModelPath() == null)
     {
@@ -258,7 +174,7 @@ public abstract class ModelImporter
   {
     if (getOriginalGenModel() != null)
     {
-      List ePackages = new EPackageList();
+      List ePackages = new ConverterUtil.EPackageList();
       for (Iterator i = getOriginalGenModel().getGenPackages().iterator(); i.hasNext();)
       {
         GenPackage genPackage = (GenPackage)i.next();
@@ -309,7 +225,7 @@ public abstract class ModelImporter
     return null;
   }
 
-  public IStatus checkGenModelFileName()
+  public Diagnostic checkGenModelFileName()
   {
     String message = null;
     String name = getGenModelFileName();
@@ -324,15 +240,15 @@ public abstract class ModelImporter
 
     if (message == null)
     {
-      return Status.OK_STATUS;
+      return Diagnostic.OK_INSTANCE;
     }
     else
     {
-      return new Status(IStatus.ERROR, ImporterPlugin.ID, ImporterUtil.ACTION_DEFAULT, message, null);
+      return new BasicDiagnostic(Diagnostic.ERROR, ConverterPlugin.ID, ConverterUtil.ACTION_DEFAULT, message, null);
     }
   }
 
-  public IStatus checkEcoreModelFileName(String fileName, String packageName)
+  public Diagnostic checkEcoreModelFileName(String fileName, String packageName)
   {
     String message = null;
 
@@ -350,11 +266,11 @@ public abstract class ModelImporter
     }
     if (message == null)
     {
-      return Status.OK_STATUS;
+      return Diagnostic.OK_INSTANCE;
     }
     else
     {
-      return new Status(IStatus.ERROR, ImporterPlugin.ID, ImporterUtil.ACTION_DEFAULT, message, null);
+      return new BasicDiagnostic(Diagnostic.ERROR, ConverterPlugin.ID, ConverterUtil.ACTION_DEFAULT, message, null);
     }
   }
 
@@ -443,47 +359,19 @@ public abstract class ModelImporter
     }
   }
 
-  protected GenModel getOriginalGenModel()
+  public GenModel getOriginalGenModel()
   {
     return originalGenModel;
   }
 
-  public List getEPackages()
+  public EPackageImportInfo getEPackageImportInfo(EPackage ePackage)
   {
-    if (ePackages == null)
-    {
-      ePackages = new EPackageList();
-    }
-    return ePackages;
+    return (EPackageImportInfo)getEPackageConvertInfo(ePackage);
   }
-
-  protected Map getEPackageToInfoMap()
+  
+  protected EPackageConvertInfo createEPackageInfo(EPackage ePackage)
   {
-    if (ePackageToInfoMap == null)
-    {
-      ePackageToInfoMap = new HashMap();
-    }
-    return ePackageToInfoMap;
-  }
-
-  public EPackageInfo getEPackageInfo(EPackage ePackage)
-  {
-    EPackageInfo ePackageInfo = (EPackageInfo)getEPackageToInfoMap().get(ePackage);
-    if (ePackageInfo == null)
-    {
-      ePackageInfo = new EPackageInfo();
-      getEPackageToInfoMap().put(ePackage, ePackageInfo);
-    }
-    return ePackageInfo;
-  }
-
-  public List getReferencedGenPackages()
-  {
-    if (referencedGenPackages == null)
-    {
-      referencedGenPackages = new GenPackageList();
-    }
-    return referencedGenPackages;
+    return new EPackageImportInfo();
   }
 
   protected GenPackage getGenPackage(EPackage ePackage)
@@ -554,116 +442,83 @@ public abstract class ModelImporter
       setModelLocation(null);
     }
   }
-
-  public EPackage getReferredEPackage(GenPackage genPackage)
+    
+  protected ResourceSet createExternalGenModelResourceSet()
   {
-    String nsURI = genPackage.getEcorePackage().getNsURI();
-    if (nsURI != null)
-    {
-      for (Iterator j = ePackages.iterator(); j.hasNext();)
-      {
-        EPackage ePackage = (EPackage)j.next();
-        if (nsURI.equals(ePackage.getNsURI()))
-        {
-          return ePackage;
-        }
-      }
-    }
-    return null;
-  }
-
-  public List filterReferencedEPackages(Collection ePackages)
-  {
-    if (ePackages.isEmpty())
-    {
-      return Collections.EMPTY_LIST;
-    }
-    else if (getReferencedGenPackages().isEmpty())
-    {
-      return new ArrayList(ePackages);
-    }
-    else
-    {
-      List filteredEPackages = new EPackageList(ePackages);
-      for (Iterator i = getReferencedGenPackages().iterator(); i.hasNext();)
-      {
-        GenPackage genPackage = (GenPackage)i.next();
-        EPackage ePackage = getReferredEPackage(genPackage);
-        if (ePackage != null)
-        {
-          filteredEPackages.remove(ePackage);
-        }
-      }
-      return filteredEPackages;
-    }
+    return getOriginalGenModel() != null ? 
+      getOriginalGenModel().eResource().getResourceSet() : 
+      super.createExternalGenModelResourceSet();
   }
   
-  public List getExternalGenModels()
+  protected void loadOriginalGenModel(URI genModelURI) throws DiagnosticException
   {
-    if (externalGenModelList == null)
+    RuntimeException runtimeException = null;
+    ResourceSet resourceSet = createResourceSet();
+    Resource resource = null;
+    try
     {
-      externalGenModelList = new UniqueEList.FastCompare();
-      if (externalGenModelResourceSet == null)
-      {
-        externalGenModelResourceSet = getOriginalGenModel() != null ? getOriginalGenModel().eResource().getResourceSet() : createResourceSet();
-      }
-      Map ePackageToGenModelMap = EcorePlugin.getEPackageNsURIToGenModelLocationMap();
-      for (Iterator i = getEPackages().iterator(); i.hasNext(); )
-      {
-        EPackage ePackage = (EPackage)i.next();
-        URI genModelURI = (URI)ePackageToGenModelMap.get(ePackage.getNsURI());
-        if (genModelURI != null)
-        {
-          try
-          {
-            Resource genModelResource = externalGenModelResourceSet.getResource(genModelURI, false);
-            if (genModelResource == null)
-            {
-              genModelResource = externalGenModelResourceSet.getResource(genModelURI, true);
-              externalGenModelList.add(genModelResource.getContents().get(0));
-            }
-          }
-          catch (Exception exception)
-          {
-            ImporterPlugin.INSTANCE.log(exception);
-          }
-        }
-      }
+      resource = resourceSet.getResource(genModelURI, true);
     }
-    return externalGenModelList;
-  }
-  
-  public ResourceSet createResourceSet()
-  {
-    return ImporterUtil.createResourceSet();
-  }
-
-  protected void loadOriginalGenModel(URI genModelURI)
-  {
-    Resource resource = createResourceSet().getResource(genModelURI, true);
-    originalGenModel = (GenModel)resource.getContents().get(0);
-
-    if (getOriginalGenModel() != null)
+    catch (RuntimeException exception)
     {
-      getOriginalGenModel().reconcile();
+      runtimeException = exception;
+      resource = resourceSet.getResource(genModelURI, false);
+    }
+
+    if (resource != null && !resource.getContents().isEmpty() && resource.getContents().get(0) instanceof GenModel)
+    {
+      originalGenModel = (GenModel)resource.getContents().get(0);
+
       setGenModelFileName(getOriginalGenModelPath().lastSegment());
       setGenModelContainerPath(getOriginalGenModelPath().removeLastSegments(1));
       genModelPath = getOriginalGenModelPath();
-    }
 
-    for (Iterator i = getOriginalGenModel().getUsedGenPackages().iterator(); i.hasNext();)
+      getOriginalGenModel().reconcile();
+      for (Iterator i = getOriginalGenModel().getUsedGenPackages().iterator(); i.hasNext();)
+      {
+        GenPackage referencedGenPackage = (GenPackage)i.next();
+        if (!referencedGenPackage.eIsProxy())
+        {
+          getReferencedGenPackages().add(referencedGenPackage);
+        }
+      }
+
+      handleOriginalGenModel();
+
+      Diagnostic diagnostic = getOriginalGenModel().diagnose();
+      if (diagnostic.getSeverity() != Diagnostic.OK)
+      {
+        throw new DiagnosticException(diagnostic);
+      }
+    }
+    else
     {
-      GenPackage referencedGenPackage = (GenPackage)i.next();
-      getReferencedGenPackages().add(referencedGenPackage);
+      Diagnostic diagnostic = runtimeException != null ?
+        BasicDiagnostic.toDiagnostic(runtimeException) :
+        new BasicDiagnostic(
+          Diagnostic.ERROR, 
+          ImporterPlugin.ID, 
+          0, 
+          ImporterPlugin.INSTANCE.getString("_UI_LoadProblem_title"),
+          null);        
+      throw new DiagnosticException(diagnostic);
     }
   }
 
-  public IStatus computeEPackages(IProgressMonitor progressMonitor) throws Exception
+  /**
+   * Subclasses may overwrite this method to perform actions on the reloaded genmodel.
+   * @throws DiagnosticException
+   */
+  protected void handleOriginalGenModel() throws DiagnosticException
+  {
+  }
+
+  public Diagnostic computeEPackages(Monitor monitor) throws Exception
   {
     clearEPackagesCollections();
-    IStatus status = doComputeEPackages(progressMonitor);
+    Diagnostic diagnostic = doComputeEPackages(monitor);
     presetEPackagesToGenerate();
-    return status;
+    return diagnostic;
   }
   
   protected void presetEPackagesToGenerate()
@@ -671,43 +526,24 @@ public abstract class ModelImporter
     int size = getEPackages().size(); 
     if (size == 1)
     {
-      getEPackageInfo((EPackage)getEPackages().get(0)).setGenerate(true);
+      getEPackageImportInfo((EPackage)getEPackages().get(0)).setConvert(true);
     }
     else if (size > 1)
     {
       List ePackagesBeingReloaded = computeEPackagesBeingReloaded();
       for (Iterator i = ePackagesBeingReloaded.iterator(); i.hasNext();)
       {
-        getEPackageInfo((EPackage)i.next()).setGenerate(true);
+        getEPackageImportInfo((EPackage)i.next()).setConvert(true);
       }
     }    
   }
 
-  protected IStatus doComputeEPackages(IProgressMonitor progressMonitor) throws Exception
+  protected Diagnostic doComputeEPackages(Monitor monitor) throws Exception
   {
-    return Status.OK_STATUS;
+    return Diagnostic.OK_INSTANCE;
   }
 
-  public void clearEPackagesCollections()
-  {
-    if (ePackages != null)
-    {
-      ePackages.clear();
-      ePackages = null;
-    }
-    if (ePackageToInfoMap != null)
-    {
-      ePackageToInfoMap.clear();
-      ePackageToInfoMap = null;
-    }
-    if (externalGenModelList != null)
-    {
-      externalGenModelList.clear();
-      externalGenModelList = null;
-    }
-  }
-
-  public void adjustEPackages(IProgressMonitor progressMonitor)
+  public void adjustEPackages(Monitor monitor)
   {
     TreeIterator ePackagesIterator = new AbstractTreeIterator(getEPackages(), false)
     {
@@ -722,15 +558,15 @@ public abstract class ModelImporter
     while (ePackagesIterator.hasNext())
     {
       EPackage ePackage = (EPackage)ePackagesIterator.next();
-      adjustEPackage(progressMonitor, ePackage);
+      adjustEPackage(monitor, ePackage);
     }
     
-    makeEcoreFileNamesUnique();
+    makeEPackageConvertDataUnique();
   }
 
-  protected void adjustEPackage(IProgressMonitor progressMonitor, EPackage ePackage)
+  protected void adjustEPackage(Monitor monitor, EPackage ePackage)
   {
-    EPackageInfo ePackageInfo = getEPackageInfo(ePackage);
+    EPackageImportInfo ePackageInfo = getEPackageImportInfo(ePackage);
 
     String name = ePackage.getName();
     int index = name.lastIndexOf(".");
@@ -744,7 +580,19 @@ public abstract class ModelImporter
       {
         namePackage = basePackage + "." + namePackage;
       }
-      ePackageInfo.setBasePackage(namePackage);
+
+      StringBuffer basePackageName = new StringBuffer();
+      for (StringTokenizer stringTokenizer = new StringTokenizer(namePackage, "."); stringTokenizer.hasMoreTokens(); )
+      {
+        String packageName = stringTokenizer.nextToken();
+        basePackageName.append(CodeGenUtil.safeName(packageName));
+        if (stringTokenizer.hasMoreTokens())
+        {
+          basePackageName.append('.');
+        }
+      }
+
+      ePackageInfo.setBasePackage(basePackageName.toString());
       ePackage.setName(name);
     }
 
@@ -779,7 +627,7 @@ public abstract class ModelImporter
     return workspaceRoot;
   }
 
-  public void prepareGenModelAndEPackages(IProgressMonitor progressMonitor)
+  public void prepareGenModelAndEPackages(Monitor monitor)
   {
     ResourceSet resourceSet = getGenModelResourceSet();
     
@@ -787,17 +635,19 @@ public abstract class ModelImporter
 
     // Create resources for all the root EPackages.
     //
-    List ePackages = computeEPackagesToGenerate();
+    List ePackages = computeEPackagesToConvert();
     for (Iterator i = ePackages.iterator(); i.hasNext();)
     {
       EPackage ePackage = (EPackage)i.next();
       addToResource(ePackage, resourceSet);
     }
 
+    List referencedGenPackages = computeValidReferencedGenPackages(); 
+      
     // Create resources for all the referenced EPackages
     // The referencedEPackage is a "local" instance of the realEPackage.  We 
     // will add the former o a resource that has the same URI of the later.
-    for (Iterator i = getReferencedGenPackages().iterator(); i.hasNext();)
+    for (Iterator i = referencedGenPackages.iterator(); i.hasNext();)
     {
       GenPackage genPackage = (GenPackage)i.next();
       EPackage realEPackage = genPackage.getEcorePackage();
@@ -813,9 +663,10 @@ public abstract class ModelImporter
     // Initialize the GenModel with all the computed data.
     //
     getGenModel().initialize(ePackages);
-    getGenModel().getUsedGenPackages().addAll(getReferencedGenPackages());
+    getGenModel().getUsedGenPackages().addAll(referencedGenPackages);
     traverseGenPackages(getGenModel().getGenPackages());
-    adjustGenModel(progressMonitor);
+    adjustGenModel(monitor);
+    adjustUsedGenPackages();
 
     // Restore all configured settings from the original.
     //
@@ -826,7 +677,7 @@ public abstract class ModelImporter
   {
     if (ePackage.eResource() == null)
     {
-      EPackageInfo ePackageInfo = getEPackageInfo(ePackage);
+      EPackageImportInfo ePackageInfo = getEPackageImportInfo(ePackage);
       String fileName = ePackageInfo.getEcoreFileName();
       if (fileName != null)
       {
@@ -843,7 +694,7 @@ public abstract class ModelImporter
     }
   }
 
-  public void saveGenModelAndEPackages(IProgressMonitor progressMonitor) throws Exception
+  public void saveGenModelAndEPackages(Monitor monitor) throws Exception
   {
     String projectName = getModelProjectName();
     IProject project = getWorkspaceRoot().getProject(projectName);
@@ -855,12 +706,11 @@ public abstract class ModelImporter
         GenPackage genPackage = (GenPackage)i.next();
         referencedGenModels.add(genPackage.getGenModel());
       }
-      createProject(progressMonitor, project, referencedGenModels);
+      createProject(monitor, project, referencedGenModels);
     }
 
-    
-    List resources = computeResourcesToBeSaved();
-    String readOnlyFiles = validateFiles(resources);
+    List resources = computeResourcesToBeSaved();    
+    String readOnlyFiles = ConverterUtil.WorkspaceResourceValidator.validate(resources);
     if (readOnlyFiles != null)
     {
       throw new Exception(ImporterPlugin.INSTANCE.getString("_UI_ReadOnlyFiles_error", new String[]{readOnlyFiles})); 
@@ -898,80 +748,7 @@ public abstract class ModelImporter
     return resources;
   }
   
-  /**
-   * Invokes the Platform validateEdit method for all the read-only files 
-   * referred by a given resource in the list.  Returns null if the resources
-   * can be saved or a comma separated list of the files that are read-only.  
-   * @param resources
-   * @return String
-   */
-  protected String validateFiles(List resources)
-  {
-    IWorkspace workspace = ResourcesPlugin.getWorkspace();
-    IWorkspaceRoot workspaceRoot= workspace.getRoot();
-    
-    List workspaceFiles = new ArrayList(resources.size());
-    List extenalFiles = new ArrayList(resources.size());
-    for (Iterator i = resources.iterator(); i.hasNext();)
-    {
-      Resource resource = (Resource)i.next();
-      URI uri = resource.getURI().trimFragment();
-      if (uri.isFile())
-      {
-        File file = new File(uri.toFileString());
-        if (file.isFile() && !file.canWrite())
-        {
-          extenalFiles.add(file);
-        }        
-      }
-      else if (uri.toString().startsWith("platform:/resource"))
-      {
-        String path = uri.toString().substring("platform:/resources".length());
-        IResource workspaceResource = workspaceRoot.findMember(new Path(path));
-        if (workspaceResource != null && workspaceResource.getType() == IResource.FILE && workspaceResource.getResourceAttributes().isReadOnly())
-        {
-          workspaceFiles.add(workspaceResource);
-        }
-      }
-    }
-    
-    StringBuffer readOnlyFiles = new StringBuffer();
-    if (!workspaceFiles.isEmpty())
-    {
-      Object context = null;
-      if (Platform.getBundle("org.eclipse.swt") != null)
-      {
-        context = new ShellFinder().getActiveShell();
-      }
-      
-      IFile[] files = (IFile[])workspaceFiles.toArray(new IFile [workspaceFiles.size()]);
-      IStatus status = workspace.validateEdit(files, context);
-      if (!status.isOK())
-      {
-        for (int i = 0; i < files.length; i++)
-        {
-          if (files[i].isReadOnly())
-          {
-            readOnlyFiles.append(", ").append(files[i].getFullPath().toString());
-          }
-        }
-      }
-    }
-    if (!extenalFiles.isEmpty())
-    {
-      for (Iterator i = extenalFiles.iterator(); i.hasNext();)
-      {
-        File file = (File)i.next();
-        readOnlyFiles.append(", ").append(file.getAbsolutePath());
-      }
-    }
-    
-    return readOnlyFiles.length() == 0 ? 
-      null : 
-      readOnlyFiles.deleteCharAt(0).deleteCharAt(0).toString();
-  }
-
-  protected void createProject(IProgressMonitor progressMonitor, IProject project, Collection referencedGenModels)
+  protected void createProject(Monitor monitor, IProject project, Collection referencedGenModels)
   {
     IWorkspaceRoot workspaceRoot = getWorkspaceRoot();
 
@@ -987,7 +764,7 @@ public abstract class ModelImporter
       {
         referencedModelProjects.add(workspaceRoot.getProject(new Path(modelDirectory).segment(0)));
         String editDirectory = referencedGenModel.getEditDirectory();
-        if (editDirectory != null && !modelDirectory.equals(editDirectory))
+        if (editDirectory != null && !modelDirectory.equals(editDirectory) && !"".equals(editDirectory))
         {
           referencedEditProjects.add(workspaceRoot.getProject(new Path(editDirectory).segment(0)));
         }
@@ -1014,11 +791,11 @@ public abstract class ModelImporter
       (new Path(path),
        getGenModelProjectLocation(),
        referencedProjects,
-       progressMonitor,
+       monitor,
        Generator.EMF_MODEL_PROJECT_STYLE | Generator.EMF_EMPTY_PROJECT_STYLE);
   }
 
-  protected void adjustGenModel(IProgressMonitor progressMonitor)
+  protected void adjustGenModel(Monitor monitor)
   {
     String modelName = URI.decode(getGenModelPath().removeFileExtension().lastSegment());
     int index = modelName.lastIndexOf('.');
@@ -1034,20 +811,39 @@ public abstract class ModelImporter
     genModel.setModelDirectory(getModelPluginDirectory());
     genModel.getUsedGenPackages().addAll(genModel.computeMissingUsedGenPackages());
   }
-
-  protected List computeEPackagesToGenerate()
+  
+  protected void adjustUsedGenPackages()
   {
-    List ePackages = new EPackageList();
-    for (Iterator i = getEPackages().iterator(); i.hasNext();)
+    if (getOriginalGenModel() != null && !getOriginalGenModel().getUsedGenPackages().isEmpty())
     {
-      EPackage ePackage = (EPackage)i.next();
-      EPackageInfo ePackageInfo = getEPackageInfo(ePackage);
-      if (ePackageInfo.isGenerate() && ePackageInfo.getEcoreFileName() != null)
+      GenModel genModel = getGenModel();
+      List usedGenPackages = new ArrayList(genModel.getUsedGenPackages());
+      usedGenPackages.removeAll(getOriginalGenModel().getUsedGenPackages());
+      if (!usedGenPackages.isEmpty())
       {
-        ePackages.add(ePackage);
+        Map nsURIOriginalUsedGenPackageMap = new HashMap();
+        for (Iterator i = getOriginalGenModel().getUsedGenPackages().iterator(); i.hasNext();)
+        {
+          GenPackage genPackage = (GenPackage)i.next();
+          nsURIOriginalUsedGenPackageMap.put(genPackage.getNSURI(), genPackage);
+        }
+        
+        for (Iterator i = usedGenPackages.iterator(); i.hasNext();)
+        {
+          GenPackage genPackage = (GenPackage)i.next();
+          GenPackage originalUsedGenPackage = (GenPackage)nsURIOriginalUsedGenPackageMap.get(genPackage.getNSURI());
+          if (originalUsedGenPackage != null)
+          {
+            genModel.getUsedGenPackages().remove(originalUsedGenPackage);
+          }
+        }
       }
     }
-    return filterReferencedEPackages(ePackages);
+  }
+
+  protected boolean canConvert(EPackage ePackage)
+  {
+    return super.canConvert(ePackage) && getEPackageImportInfo(ePackage).getEcoreFileName() != null;
   }
 
   public void traverseGenPackages(List genPackages)
@@ -1056,7 +852,7 @@ public abstract class ModelImporter
     {
       GenPackage genPackage = (GenPackage)i.next();
       EPackage ePackage = genPackage.getEcorePackage();
-      EPackageInfo ePackageInfo = getEPackageInfo(ePackage);
+      EPackageImportInfo ePackageInfo = getEPackageImportInfo(ePackage);
 
       genPackage.setBasePackage(ePackageInfo.getBasePackage());
       genPackage.setPrefix(ePackageInfo.getPrefix());
@@ -1157,76 +953,10 @@ public abstract class ModelImporter
     }
 
     return null;
-  }
-  
-  /**
-   * Changes the existing EPackage Infos so that no duplicated names 
-   * are used.
-   */
-  public void makeEcoreFileNamesUnique()
-  {
-    if (ePackageToInfoMap != null)
-    {
-      Map counterByEcoreName = new HashMap();
-      List ePackages = filterReferencedEPackages(ePackageToInfoMap.keySet());
-      if (!ePackages.isEmpty())
-      {
-        List ePackageInfos = new ArrayList(ePackages.size());
-        for (Iterator i = ePackages.iterator(); i.hasNext();)
-        {
-          EPackage ePackage = (EPackage)i.next();
-          if (ePackage.getESuperPackage() == null || !ePackages.contains(ePackage.getESuperPackage()))
-          {
-            EPackageInfo ePackageInfo = (EPackageInfo)ePackageToInfoMap.get(ePackage);
-            ePackageInfos.add(ePackageInfo);
-            String fileName = ePackageInfo.getEcoreFileName();
-            if (fileName != null)
-            {
-              counterByEcoreName.put(fileName, null);
-            }
-          }
-        }
-        
-        for (Iterator i = ePackageInfos.iterator(); i.hasNext();)
-        {        
-          EPackageInfo ePackageInfo = (EPackageInfo)i.next();
-          String fileName = ePackageInfo.getEcoreFileName();
-          if (fileName != null)
-          {
-            Integer counterObject = (Integer)counterByEcoreName.get(fileName);
-            if (counterObject != null)
-            {
-              int counter = counterObject.intValue();
-              int index = fileName.lastIndexOf(".");
-              StringBuffer newFileName = null;
-              do
-              {            
-                newFileName = new StringBuffer(fileName).insert(index, counter++);
-              }
-              while (counterByEcoreName.containsKey(newFileName.toString()));
-              
-              ePackageInfo.setEcoreFileName(newFileName.toString());
-              counterObject = new Integer(counter);
-              counterByEcoreName.put(newFileName.toString(), new Integer(1));
-            }
-            else
-            {
-              counterObject = new Integer(1);
-            }        
-            counterByEcoreName.put(fileName, counterObject);
-          }
-        }
-      }
-    }
-  }
-
+  }  
+      
   protected Map getEcoreSaveOptions()
   {
     return Collections.EMPTY_MAP;
   }
-
-  protected Map getGenmodelSaveOptions()
-  {
-    return Collections.EMPTY_MAP;
-  }
 }
