@@ -3,16 +3,16 @@
  *
  * Copyright (c) 2002-2004 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
+ * http://www.eclipse.org/legal/epl-v10.html
  * 
  * Contributors: 
  *   IBM - Initial API and implementation
  *
  * </copyright>
  *
- * $Id: XSDAttributeGroupDefinitionImpl.java,v 1.3 2004/08/11 15:08:54 marcelop Exp $
+ * $Id: XSDAttributeGroupDefinitionImpl.java,v 1.11 2005/11/25 13:14:00 emerks Exp $
  */
 package org.eclipse.xsd.impl;
 
@@ -33,7 +33,6 @@ import org.eclipse.emf.common.notify.NotificationChain;
 import org.eclipse.emf.common.util.EList;
 import org.eclipse.emf.ecore.EClass;
 import org.eclipse.emf.ecore.EReference;
-import org.eclipse.emf.ecore.EStructuralFeature;
 import org.eclipse.emf.ecore.InternalEObject;
 import org.eclipse.emf.ecore.impl.ENotificationImpl;
 import org.eclipse.emf.ecore.util.EObjectContainmentEList;
@@ -176,7 +175,12 @@ public class XSDAttributeGroupDefinitionImpl
 
   protected int analysisState;
 
-  protected XSDAttributeGroupDefinitionImpl() 
+  /**
+   * <!-- begin-user-doc -->
+   * <!-- end-user-doc -->
+   * @generated NOT
+   */
+  protected XSDAttributeGroupDefinitionImpl()
   {
     super();
     this.resolvedAttributeGroupDefinition = this;
@@ -190,7 +194,7 @@ public class XSDAttributeGroupDefinitionImpl
    */
   protected EClass eStaticClass()
   {
-    return XSDPackage.eINSTANCE.getXSDAttributeGroupDefinition();
+    return XSDPackage.Literals.XSD_ATTRIBUTE_GROUP_DEFINITION;
   }
 
   /**
@@ -233,7 +237,8 @@ public class XSDAttributeGroupDefinitionImpl
     analysisState = UNANALYZED;
     super.patch();
     XSDAttributeGroupDefinition theResolvedAttributeGroupDefinition = getResolvedAttributeGroupDefinition();
-    if (theResolvedAttributeGroupDefinition != this && theResolvedAttributeGroupDefinition.getContainer() == null)
+    if (theResolvedAttributeGroupDefinition != this && 
+          (forceResolve || theResolvedAttributeGroupDefinition.getContainer() == null))
     {
       XSDAttributeGroupDefinition newResolvedAttributeGroupDefinition =
         resolveAttributeGroupDefinition
@@ -588,7 +593,7 @@ public class XSDAttributeGroupDefinitionImpl
 
   protected void handleReconciliation(List newContents, List remainingContents)
   {
-    handleAnnotationReconciliation(XSDPackage.eINSTANCE.getXSDAttributeGroupDefinition_Annotation(), newContents, remainingContents);
+    handleAnnotationReconciliation(XSDPackage.Literals.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION, newContents, remainingContents);
   
     XSDWildcard newAttributeWildcard = null;
     if (!newContents.isEmpty())
@@ -645,7 +650,7 @@ public class XSDAttributeGroupDefinitionImpl
     Element theElement = getElement();
     if (theElement != null)
     {
-      if (eReference == null || eReference == XSDPackage.eINSTANCE.getXSDAttributeGroupDefinition_ResolvedAttributeGroupDefinition())
+      if (eReference == null || eReference == XSDPackage.Literals.XSD_ATTRIBUTE_GROUP_DEFINITION__RESOLVED_ATTRIBUTE_GROUP_DEFINITION)
       {
         XSDAttributeGroupDefinition theResolvedAttributeGroupDefinition = getResolvedAttributeGroupDefinition();
         if (theResolvedAttributeGroupDefinition != this)
@@ -659,7 +664,7 @@ public class XSDAttributeGroupDefinitionImpl
   protected void adoptContent(EReference eReference, XSDConcreteComponent xsdConcreteComponent)
   {
     super.adoptContent(eReference, xsdConcreteComponent);
-    if (eReference == XSDPackage.eINSTANCE.getXSDAttributeGroupDefinition_Contents())
+    if (eReference == XSDPackage.Literals.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTENTS)
     {
       traverseToRootForPatching();
     }
@@ -668,7 +673,7 @@ public class XSDAttributeGroupDefinitionImpl
   protected void orphanContent(EReference eReference, XSDConcreteComponent xsdConcreteComponent)
   {
     super.orphanContent(eReference, xsdConcreteComponent);
-    if (eReference == XSDPackage.eINSTANCE.getXSDAttributeGroupDefinition_Contents())
+    if (eReference == XSDPackage.Literals.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTENTS)
     {
       traverseToRootForPatching();
     }
@@ -902,27 +907,20 @@ public class XSDAttributeGroupDefinitionImpl
    * <!-- end-user-doc -->
    * @generated
    */
-  public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, Class baseClass, NotificationChain msgs)
+  public NotificationChain eInverseRemove(InternalEObject otherEnd, int featureID, NotificationChain msgs)
   {
-    if (featureID >= 0)
+    switch (featureID)
     {
-      switch (eDerivedStructuralFeatureID(featureID, baseClass))
-      {
-        case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__DIAGNOSTICS:
-          return ((InternalEList)getDiagnostics()).basicRemove(otherEnd, msgs);
-        case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
-          return basicSetAnnotation(null, msgs);
-        case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTENTS:
-          return ((InternalEList)getContents()).basicRemove(otherEnd, msgs);
-        case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ATTRIBUTE_WILDCARD_CONTENT:
-          return basicSetAttributeWildcardContent(null, msgs);
-        case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SYNTHETIC_WILDCARD:
-          return basicSetSyntheticWildcard(null, msgs);
-        default:
-          return eDynamicInverseRemove(otherEnd, featureID, baseClass, msgs);
-      }
+      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
+        return basicSetAnnotation(null, msgs);
+      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTENTS:
+        return ((InternalEList)getContents()).basicRemove(otherEnd, msgs);
+      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ATTRIBUTE_WILDCARD_CONTENT:
+        return basicSetAttributeWildcardContent(null, msgs);
+      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SYNTHETIC_WILDCARD:
+        return basicSetSyntheticWildcard(null, msgs);
     }
-    return eBasicSetContainer(null, featureID, msgs);
+    return super.eInverseRemove(otherEnd, featureID, msgs);
   }
 
   /**
@@ -930,34 +928,10 @@ public class XSDAttributeGroupDefinitionImpl
    * <!-- end-user-doc -->
    * @generated
    */
-  public Object eGet(EStructuralFeature eFeature, boolean resolve)
+  public Object eGet(int featureID, boolean resolve, boolean coreType)
   {
-    switch (eDerivedStructuralFeatureID(eFeature))
+    switch (featureID)
     {
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ELEMENT:
-        return getElement();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTAINER:
-        return getContainer();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ROOT_CONTAINER:
-        return getRootContainer();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SCHEMA:
-        return getSchema();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__DIAGNOSTICS:
-        return getDiagnostics();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__NAME:
-        return getName();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__TARGET_NAMESPACE:
-        return getTargetNamespace();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ALIAS_NAME:
-        return getAliasName();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__URI:
-        return getURI();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ALIAS_URI:
-        return getAliasURI();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__QNAME:
-        return getQName();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CIRCULAR:
-        return isCircular() ? Boolean.TRUE : Boolean.FALSE;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ATTRIBUTE_GROUP_DEFINITION_REFERENCE:
         return isAttributeGroupDefinitionReference() ? Boolean.TRUE : Boolean.FALSE;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
@@ -975,7 +949,7 @@ public class XSDAttributeGroupDefinitionImpl
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SYNTHETIC_WILDCARD:
         return getSyntheticWildcard();
     }
-    return eDynamicGet(eFeature, resolve);
+    return super.eGet(featureID, resolve, coreType);
   }
 
   /**
@@ -983,23 +957,10 @@ public class XSDAttributeGroupDefinitionImpl
    * <!-- end-user-doc -->
    * @generated
    */
-  public void eSet(EStructuralFeature eFeature, Object newValue)
+  public void eSet(int featureID, Object newValue)
   {
-    switch (eDerivedStructuralFeatureID(eFeature))
+    switch (featureID)
     {
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ELEMENT:
-        setElement((Element)newValue);
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__DIAGNOSTICS:
-        getDiagnostics().clear();
-        getDiagnostics().addAll((Collection)newValue);
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__NAME:
-        setName((String)newValue);
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__TARGET_NAMESPACE:
-        setTargetNamespace((String)newValue);
-        return;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
         setAnnotation((XSDAnnotation)newValue);
         return;
@@ -1024,7 +985,7 @@ public class XSDAttributeGroupDefinitionImpl
         setSyntheticWildcard((XSDWildcard)newValue);
         return;
     }
-    eDynamicSet(eFeature, newValue);
+    super.eSet(featureID, newValue);
   }
 
   /**
@@ -1032,22 +993,10 @@ public class XSDAttributeGroupDefinitionImpl
    * <!-- end-user-doc -->
    * @generated
    */
-  public void eUnset(EStructuralFeature eFeature)
+  public void eUnset(int featureID)
   {
-    switch (eDerivedStructuralFeatureID(eFeature))
+    switch (featureID)
     {
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ELEMENT:
-        setElement(ELEMENT_EDEFAULT);
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__DIAGNOSTICS:
-        getDiagnostics().clear();
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__NAME:
-        setName(NAME_EDEFAULT);
-        return;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__TARGET_NAMESPACE:
-        setTargetNamespace(TARGET_NAMESPACE_EDEFAULT);
-        return;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
         setAnnotation((XSDAnnotation)null);
         return;
@@ -1070,7 +1019,7 @@ public class XSDAttributeGroupDefinitionImpl
         setSyntheticWildcard((XSDWildcard)null);
         return;
     }
-    eDynamicUnset(eFeature);
+    super.eUnset(featureID);
   }
 
   /**
@@ -1078,34 +1027,10 @@ public class XSDAttributeGroupDefinitionImpl
    * <!-- end-user-doc -->
    * @generated
    */
-  public boolean eIsSet(EStructuralFeature eFeature)
+  public boolean eIsSet(int featureID)
   {
-    switch (eDerivedStructuralFeatureID(eFeature))
+    switch (featureID)
     {
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ELEMENT:
-        return ELEMENT_EDEFAULT == null ? element != null : !ELEMENT_EDEFAULT.equals(element);
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CONTAINER:
-        return getContainer() != null;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ROOT_CONTAINER:
-        return getRootContainer() != null;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SCHEMA:
-        return getSchema() != null;
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__DIAGNOSTICS:
-        return diagnostics != null && !diagnostics.isEmpty();
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__NAME:
-        return NAME_EDEFAULT == null ? name != null : !NAME_EDEFAULT.equals(name);
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__TARGET_NAMESPACE:
-        return TARGET_NAMESPACE_EDEFAULT == null ? targetNamespace != null : !TARGET_NAMESPACE_EDEFAULT.equals(targetNamespace);
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ALIAS_NAME:
-        return ALIAS_NAME_EDEFAULT == null ? getAliasName() != null : !ALIAS_NAME_EDEFAULT.equals(getAliasName());
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__URI:
-        return URI_EDEFAULT == null ? getURI() != null : !URI_EDEFAULT.equals(getURI());
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ALIAS_URI:
-        return ALIAS_URI_EDEFAULT == null ? getAliasURI() != null : !ALIAS_URI_EDEFAULT.equals(getAliasURI());
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__QNAME:
-        return QNAME_EDEFAULT == null ? getQName() != null : !QNAME_EDEFAULT.equals(getQName());
-      case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__CIRCULAR:
-        return isCircular() != CIRCULAR_EDEFAULT;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ATTRIBUTE_GROUP_DEFINITION_REFERENCE:
         return isAttributeGroupDefinitionReference() != ATTRIBUTE_GROUP_DEFINITION_REFERENCE_EDEFAULT;
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__ANNOTATION:
@@ -1123,7 +1048,7 @@ public class XSDAttributeGroupDefinitionImpl
       case XSDPackage.XSD_ATTRIBUTE_GROUP_DEFINITION__SYNTHETIC_WILDCARD:
         return syntheticWildcard != null;
     }
-    return eDynamicIsSet(eFeature);
+    return super.eIsSet(featureID);
   }
 
   public String getQName()
@@ -1179,13 +1104,19 @@ public class XSDAttributeGroupDefinitionImpl
     {
       if (getAnnotation() != null)
       {
-        clonedAttributeGroupDefinition.setAnnotation((XSDAnnotation)getAnnotation().cloneConcreteComponent(deep, shareDOM));
+        clonedAttributeGroupDefinition.setAnnotation((XSDAnnotation)getAnnotation().cloneConcreteComponent(true, shareDOM));
       }
 
       if (!getContents().isEmpty())
       {
         clonedAttributeGroupDefinition.getContents().addAll(cloneConcreteComponents(getContents(), true, shareDOM));
       }
+
+      if (getAttributeWildcardContent() != null)
+      {
+        clonedAttributeGroupDefinition.setAttributeWildcardContent
+          ((XSDWildcard)getAttributeWildcardContent().cloneConcreteComponent(true, shareDOM));
+      }
     }
 
     if (shareDOM && getElement() != null)
