@@ -1,7 +1,7 @@
 /**
  * <copyright>
  *
- * Copyright (c) 2002-2004 IBM Corporation and others.
+ * Copyright (c) 2002-2006 IBM Corporation and others.
  * All rights reserved.   This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -12,7 +12,7 @@
  *
  * </copyright>
  *
- * $Id: XSDResourceImpl.java,v 1.9 2005/06/24 23:05:52 emerks Exp $
+ * $Id: XSDResourceImpl.java,v 1.16 2006/06/13 21:13:26 emerks Exp $
  */
 package org.eclipse.xsd.util;
 
@@ -20,23 +20,39 @@ package org.eclipse.xsd.util;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.OutputStreamWriter;
+import java.io.Writer;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
 import javax.xml.parsers.DocumentBuilder;
-import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.transform.OutputKeys;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.TransformerException;
-import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.stream.StreamResult;
 
+import org.eclipse.core.runtime.IProgressMonitor;
+import org.eclipse.emf.common.notify.Notification;
+import org.eclipse.emf.common.notify.impl.AdapterImpl;
+import org.eclipse.emf.common.util.URI;
+import org.eclipse.emf.ecore.EObject;
+import org.eclipse.emf.ecore.resource.Resource;
+import org.eclipse.emf.ecore.resource.ResourceSet;
+import org.eclipse.emf.ecore.resource.URIConverter;
+import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
+import org.eclipse.xsd.XSDConcreteComponent;
+import org.eclipse.xsd.XSDDiagnostic;
+import org.eclipse.xsd.XSDDiagnosticSeverity;
+import org.eclipse.xsd.XSDFactory;
+import org.eclipse.xsd.XSDPlugin;
+import org.eclipse.xsd.XSDSchema;
+import org.eclipse.xsd.impl.XSDSchemaImpl;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -46,23 +62,6 @@ import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
 
-import org.eclipse.core.runtime.IProgressMonitor;
-
-import org.eclipse.emf.common.notify.impl.AdapterImpl;
-import org.eclipse.emf.common.util.URI;
-import org.eclipse.emf.ecore.EObject;
-import org.eclipse.emf.ecore.resource.Resource;
-import org.eclipse.emf.ecore.resource.ResourceSet;
-import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
-
-import org.eclipse.xsd.XSDConcreteComponent;
-import org.eclipse.xsd.XSDDiagnostic;
-import org.eclipse.xsd.XSDDiagnosticSeverity;
-import org.eclipse.xsd.XSDFactory;
-import org.eclipse.xsd.XSDPlugin;
-import org.eclipse.xsd.XSDSchema;
-import org.eclipse.xsd.impl.XSDSchemaImpl;
-
 
 /**
  * The <b>Resource</b> implementation for the model.
@@ -77,6 +76,22 @@ public class XSDResourceImpl extends ResourceImpl
   public static String XSD_PROGRESS_MONITOR = "XSD_PROGRESS_MONITOR";
 
   public static String XSD_ENCODING = "XSD_ENCODING";
+  
+  /**
+   * This option can be used as an option on Resource#load methods to specify JAXP pool to be used
+   * for loading and serializing XML Schemas.
+   * @see Resource#load(InputStream, Map)
+   * @see Resource#load(Map)
+   */
+  public static String XSD_JAXP_POOL = "XSD_JAXP_POOL";
+  
+  /**
+   * This option can be used as an option on Resource#load methods to specify JAXP configuration
+   * that creates and configures SAX, DOM parsers and Transformer.
+   * @see Resource#load(InputStream, Map)
+   * @see Resource#load(Map)
+   */
+  public static String XSD_JAXP_CONFIG = "XSD_JAXP_CONFIG";
 
   public static class SchemaLocator extends AdapterImpl implements XSDSchemaLocator
   {
@@ -134,25 +149,9 @@ public class XSDResourceImpl extends ResourceImpl
   {
     try
     {
-      TransformerFactory transformerFactory = TransformerFactory.newInstance();
-      Transformer transformer = transformerFactory.newTransformer();
-
-      transformer.setOutputProperty(OutputKeys.INDENT, "yes");
-      transformer.setOutputProperty(OutputKeys.METHOD, "xml");
-
-      // Unless a width is set, there will be only line breaks but no indentation.
-      // The IBM JDK and the Sun JDK don't agree on the property name,
-      // so we set them both.
-      //
-      transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "2");
-      transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
-
-      if (encoding != null)
-      {
-        transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
-      }
-
-      transformer.transform(new DOMSource(element), new StreamResult(outputStream));
+      Transformer transformer = new DefaultJAXPConfiguration().createTransformer(encoding);
+      Writer writer = encoding == null ? new OutputStreamWriter(outputStream) : new OutputStreamWriter(outputStream, encoding);
+      transformer.transform(new DOMSource(element), new StreamResult(writer));
     }
     catch (TransformerException exception)
     {
@@ -177,43 +176,145 @@ public class XSDResourceImpl extends ResourceImpl
     }
   }
 
+  /**
+   * @deprecated since 2.2
+   * @see #doSerialize(OutputStream, Document, Map)
+   * @param outputStream
+   * @param document
+   * @see #doSerialize(OutputStream, Document, Map)
+   * @throws IOException
+   */
   protected static void doSerialize(OutputStream outputStream, Document document) throws IOException
   {
-    doSerialize(outputStream, document, null);
+    doSerialize(outputStream, document, Collections.EMPTY_MAP);
+  }
+  
+  protected static void doSerialize(OutputStream outputStream, Document document, Map options) throws IOException
+  {
+    JAXPPool jaxpPool = null;
+    JAXPConfiguration config = null;
+    String encoding = null;
+    if (options != null)
+    {
+      jaxpPool = (JAXPPool)options.get(XSDResourceImpl.XSD_JAXP_POOL);
+      config = (JAXPConfiguration)options.get(XSDResourceImpl.XSD_JAXP_CONFIG);
+      encoding = (String)options.get(XSD_ENCODING);
+    }
+    
+    if (jaxpPool == null)
+    {
+      if (config == null)
+      {
+        doSerialize(outputStream, document, encoding);
+      }
+      else
+      {
+        try
+        {
+          Writer writer = encoding == null ? new OutputStreamWriter(outputStream) : new OutputStreamWriter(outputStream, encoding);
+          config.createTransformer(encoding).transform(new DOMSource(document), new StreamResult(writer));
+        }
+        catch (TransformerException exception)
+        {
+          XSDPlugin.INSTANCE.log(exception);
+        }
+      }
+    }
+    else
+    {
+      Transformer transformer = null;
+      try
+      {
+        transformer = jaxpPool.getTransformer(encoding);
+        Writer writer = encoding == null ? new OutputStreamWriter(outputStream) : new OutputStreamWriter(outputStream, encoding);
+        transformer.transform(new DOMSource(document), new StreamResult(writer));
+      }
+      catch (TransformerException exception)
+      {
+        XSDPlugin.INSTANCE.log(exception);
+      }
+      finally
+      {
+        jaxpPool.releaseTransformer(transformer);
+      }
+    }
   }
 
+  /**
+   * @deprecated since 2.2
+   * @see #doSerialize(OutputStream, Document, Map)
+   * @param outputStream
+   * @param document
+   * @param encoding
+   * @throws IOException
+   */
   protected static void doSerialize(OutputStream outputStream, Document document, String encoding) throws IOException
   {
     try
     {
-      TransformerFactory transformerFactory = TransformerFactory.newInstance();
-      Transformer transformer = transformerFactory.newTransformer();
-
-      transformer.setOutputProperty(OutputKeys.INDENT, "yes");
-      transformer.setOutputProperty(OutputKeys.METHOD, "xml");
-
-      // Unless a width is set, there will be only line breaks but no indentation.
-      // The IBM JDK and the Sun JDK don't agree on the property name,
-      // so we set them both.
-      //
-      transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "2");
-      transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
-
-      if (encoding != null)
-      {
-        transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
-      }
-
-      transformer.transform(new DOMSource(document), new StreamResult(outputStream));
+      Transformer transformer = new DefaultJAXPConfiguration().createTransformer(encoding);
+      Writer writer = encoding == null ? new OutputStreamWriter(outputStream) : new OutputStreamWriter(outputStream, encoding);
+      transformer.transform(new DOMSource(document), new StreamResult(writer));
     }
     catch (TransformerException exception)
     {
       XSDPlugin.INSTANCE.log(exception);
     }
   }
+  
+  protected static void doSerialize(Writer writer, Document document, Map options) throws IOException
+  {
+    JAXPPool jaxpPool = null;
+    JAXPConfiguration config = null;
+    String encoding = null;
+    if (options != null)
+    {
+      jaxpPool = (JAXPPool)options.get(XSDResourceImpl.XSD_JAXP_POOL);
+      config = (JAXPConfiguration)options.get(XSDResourceImpl.XSD_JAXP_CONFIG);
+      encoding = (String)options.get(XSD_ENCODING);
+    }
+
+    if (jaxpPool == null)
+    {
+      try
+      {
+        if (config == null)
+        {
+          new DefaultJAXPConfiguration().createTransformer(encoding).transform(new DOMSource(document), new StreamResult(writer));
+        }
+        else
+        {
+          config.createTransformer(encoding).transform(new DOMSource(document), new StreamResult(writer));
+        }
+      }
+      catch (TransformerException exception)
+      {
+        XSDPlugin.INSTANCE.log(exception);
+      }
+    }
+    else
+    {
+      Transformer transformer = null;
+      try
+      {
+        transformer = jaxpPool.getTransformer(encoding);
+        transformer.transform(new DOMSource(document), new StreamResult(writer));
+      }
+      catch (TransformerException exception)
+      {
+        XSDPlugin.INSTANCE.log(exception);
+      }
+      finally
+      {
+        jaxpPool.releaseTransformer(transformer);
+      }
+    }
+  }
 
   /**
-   * Builds a document using JAXP.
+   * Builds a document using default JAXP.
+   * @deprecated since 2.2
+   * @see #getDocument(InputSource, ErrorHandler, Map)
    * @param inputSource the contents to parse.
    * @param errorHandler the handled used by the parser.
    * @return a document.
@@ -222,19 +323,7 @@ public class XSDResourceImpl extends ResourceImpl
   {
     try
     {
-      DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
-      documentBuilderFactory.setNamespaceAware(true);
-      documentBuilderFactory.setValidating(false);
-
-      DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
-  
-      // Create a catalog with an entity mapping URL for the cached DTD for XML Schemas and set to into the build.
-      //
-      EntityResolver entityResolver = createEntityResolver();
-      documentBuilder.setEntityResolver(entityResolver);
-
-      documentBuilder.setErrorHandler(errorHandler);
-
+      DocumentBuilder documentBuilder = new DefaultJAXPConfiguration().createDocumentBuilder(errorHandler);
       Document document = documentBuilder.parse(inputSource);
       return document;
     }
@@ -249,7 +338,77 @@ public class XSDResourceImpl extends ResourceImpl
   }
   
   /**
-   * Builds a document using JAXP.
+   * Builds DOM document using JAXP DocumentBuilder
+   * @see #XSD_JAXP_POOL
+   * @see #XSD_JAXP_CONFIG
+   * @param inputSource the content to parser
+   * @param errorHandler error handler for recording any loading errors
+   * @param options loading options
+   * @return document DOM document
+   * @throws IOException
+   */
+  protected static Document getDocument(InputSource inputSource, ErrorHandler errorHandler, Map options) throws IOException
+  {
+    JAXPPool jaxpPool = null;
+    JAXPConfiguration config = null;
+    if (options != null)
+    {
+      jaxpPool = (JAXPPool)options.get(XSDResourceImpl.XSD_JAXP_POOL);
+      config = (JAXPConfiguration)options.get(XSDResourceImpl.XSD_JAXP_CONFIG);
+    }
+
+    if (jaxpPool == null)
+    {
+      if (config == null)
+      {
+        return getDocument(inputSource, errorHandler);
+      }
+      else
+      {
+        try
+        {
+          DocumentBuilder documentBuilder = config.createDocumentBuilder(errorHandler);
+          Document document = documentBuilder.parse(inputSource);
+          return document;
+        }
+        catch (ParserConfigurationException exception)
+        {
+          throw new IOWrappedException(exception);
+        }
+        catch (SAXException exception)
+        {
+          throw new IOWrappedException(exception);
+        }
+      }
+    }
+    else
+    {
+      DocumentBuilder documentBuilder = null;
+      try
+      {
+        documentBuilder = jaxpPool.getDocumentBuilder(errorHandler);
+        Document document = documentBuilder.parse(inputSource);
+        return document;
+      }
+      catch (ParserConfigurationException exception)
+      {
+        throw new IOWrappedException(exception);
+      }
+      catch (SAXException exception)
+      {
+        throw new IOWrappedException(exception);
+      }
+      finally
+      {
+        jaxpPool.releaseDocumentBuilder(documentBuilder);
+      }
+    }
+  }
+  
+  /**
+   * @deprecated since 2.2
+   * @see #getDocument(InputSource, ErrorHandler, Map)
+   * Builds a document using default JAXP implementation.
    * @param inputStream the contents to parse.
    * @param errorHandler the handled used by the parser.
    * @return a document.
@@ -275,11 +434,16 @@ public class XSDResourceImpl extends ResourceImpl
         public InputSource resolveEntity(String publicId, String systemId) throws SAXException
         {
           InputSource inputSource;
-          if ("-//W3C//DTD XMLSCHEMA 200102//EN".equals(publicId) || "http://www.w3.org/2001/XMLSchema.dtd".equals(systemId))
+          if ("-//W3C//DTD XMLSCHEMA 200102//EN".equalsIgnoreCase(publicId) || "http://www.w3.org/2001/XMLSchema.dtd".equalsIgnoreCase(systemId))
           {
             inputSource = new InputSource(baseURL + "cache/www.w3.org/2001/XMLSchema.dtd");
             inputSource.setPublicId(publicId);
           }
+          else if (systemId != null && systemId.startsWith("file://bundleentry:"))
+          {
+            inputSource = new InputSource(systemId.substring(7));
+            inputSource.setPublicId(publicId);
+          }
           else
           {
             try
@@ -338,6 +502,59 @@ public class XSDResourceImpl extends ResourceImpl
   }
 
   protected void doSave(OutputStream os, Map options) throws IOException
+  {
+    if (os instanceof URIConverter.WriteableOutputStream)
+    {
+      doSave(((URIConverter.WriteableOutputStream)os).asWriter(), options);
+    }
+    else
+    {
+      XSDSchema xsdSchema = getSchema();
+      if (xsdSchema != null)
+      {
+        Document document = xsdSchema.getDocument();
+        if (document == null)
+        {
+          xsdSchema.updateDocument();
+          document = xsdSchema.getDocument();
+        }
+  
+        if (xsdSchema.getElement() == null)
+        {
+          xsdSchema.updateElement();
+        }
+  
+        doSerialize(os, document, options);
+      }
+    }
+  }
+
+  /**
+   * Saves the resource to the writer using the specified options.
+   * @param writer the writer
+   * @param options the save options.
+   */
+  public final void save(Writer writer, Map options) throws IOException
+  {
+    if (defaultSaveOptions == null || defaultSaveOptions.isEmpty())
+    {
+      doSave(writer, options);
+    }
+    else if (options == null)
+    {
+      doSave(writer, defaultSaveOptions);
+    }
+    else
+    {
+      Map mergedOptions = new HashMap(defaultSaveOptions);
+      mergedOptions.putAll(options);
+      doSave(writer, mergedOptions);
+    }
+    setModified(false);
+  }
+
+  
+  protected void doSave(Writer writer, Map options) throws IOException
   {
     XSDSchema xsdSchema = getSchema();
     if (xsdSchema != null)
@@ -353,11 +570,10 @@ public class XSDResourceImpl extends ResourceImpl
       {
         xsdSchema.updateElement();
       }
-
-      doSerialize(os, document, options == null ? null : (String)options.get(XSD_ENCODING));
+      doSerialize(writer, document, options);
     }
   }
-
+  
   /**
    * This gets the resource's schema.
    */
@@ -368,14 +584,57 @@ public class XSDResourceImpl extends ResourceImpl
         (XSDSchema)getContents().get(0) :
         null;
   }
+  
+  public final void load(InputSource inputSource, Map options) throws IOException
+  {
+    if (!isLoaded)
+    {
+      Notification notification = setLoaded(true);
+      isLoading = true;
 
-  /**
-   * Loads a new {@link XSDResourceImpl} into the resource set.
-   * @param inputStream the contents of the new resource.
-   * @param options any options to influence loading behavior.
-   * @return a new XSDResourceImpl.
-   */
-  protected void doLoad(InputStream inputStream, Map options) throws IOException
+      if (errors != null)
+      {
+        errors.clear();
+      }
+
+      if (warnings != null)
+      {
+        warnings.clear();
+      }
+
+      try
+      {
+        if (defaultLoadOptions == null || defaultLoadOptions.isEmpty())
+        {
+          doLoad(inputSource, options);
+        }
+        else if (options == null)
+        {
+          doLoad(inputSource, defaultLoadOptions);
+        }
+        else
+        {
+          Map mergedOptions = new HashMap(defaultLoadOptions);
+          mergedOptions.putAll(options);
+  
+          doLoad(inputSource, mergedOptions);
+        }
+      }
+      finally
+      {
+        isLoading = false;
+
+        if (notification != null)
+        {
+          eNotify(notification);
+        }
+  
+        setModified(false);
+      } 
+    }
+  }
+  
+  protected void doLoad(InputSource inputSource, Map options) throws IOException
   {
     attachedSchemas = new ArrayList();
 
@@ -391,30 +650,27 @@ public class XSDResourceImpl extends ResourceImpl
       progressMonitor.subTask(getURI().toString());
     }
 
-    XSDParser xsdParser = new XSDParser();
+    Collection errors = null;
     try
     {
       Document document;
-      InputSource inputSource = new InputSource(inputStream);
-      if (getURI() != null)
-      {
-        String id = getURI().toString();
-        inputSource.setPublicId(id);
-        inputSource.setSystemId(id);
-      }
+
       if (options != null && Boolean.TRUE.equals(options.get("XSD_TRACK_LOCATION")))
       {
+        XSDParser xsdParser = new XSDParser(options);
         xsdParser.parse(inputSource);
         document = xsdParser.getDocument();
+        if (xsdParser.getEncoding() != null)
+        {
+          getDefaultSaveOptions().put(XSD_ENCODING, xsdParser.getEncoding());
+        }
+        errors = xsdParser.getDiagnostics();
       }
       else
       {
-        document = getDocument(inputSource, xsdParser);
-      }
-
-      if (xsdParser.getEncoding() != null)
-      {
-        getDefaultSaveOptions().put(XSD_ENCODING, xsdParser.getEncoding());
+        DefaultErrorHandler errorHandler = new DefaultErrorHandler();
+        document = getDocument(inputSource, errorHandler, options);
+        errors = errorHandler.getDiagnostics();
       }
 
       if (document != null && document.getDocumentElement() != null)
@@ -449,27 +705,30 @@ public class XSDResourceImpl extends ResourceImpl
       handleSchemaElement(null, false);
     }
 
-    for (Iterator i = getContents().iterator(); i.hasNext(); )
+    if (errors != null)
     {
-      XSDSchema xsdSchema = (XSDSchema)i.next();
-      assignDiagnostics(xsdSchema, xsdParser.getDiagnostics());
-
-      for (Iterator diagnostics = xsdParser.getDiagnostics().iterator(); diagnostics.hasNext(); )
+      for (Iterator i = getContents().iterator(); i.hasNext();)
       {
-        XSDDiagnostic xsdDiagnostic = (XSDDiagnostic)diagnostics.next();
-        switch (xsdDiagnostic.getSeverity().getValue())
+        XSDSchema xsdSchema = (XSDSchema)i.next();
+        assignDiagnostics(xsdSchema, errors);
+  
+        for (Iterator diagnostics = errors.iterator(); diagnostics.hasNext();)
         {
-          case XSDDiagnosticSeverity.FATAL:
-          case XSDDiagnosticSeverity.ERROR:
+          XSDDiagnostic xsdDiagnostic = (XSDDiagnostic)diagnostics.next();
+          switch (xsdDiagnostic.getSeverity().getValue())
           {
-            getErrors().add(xsdDiagnostic);
-            break;
-          }
-          case XSDDiagnosticSeverity.WARNING:
-          case XSDDiagnosticSeverity.INFORMATION:
-          {
-            getWarnings().add(xsdDiagnostic);
-            break;
+            case XSDDiagnosticSeverity.FATAL:
+            case XSDDiagnosticSeverity.ERROR:
+            {
+              getErrors().add(xsdDiagnostic);
+              break;
+            }
+            case XSDDiagnosticSeverity.WARNING:
+            case XSDDiagnosticSeverity.INFORMATION:
+            {
+              getWarnings().add(xsdDiagnostic);
+              break;
+            }
           }
         }
       }
@@ -483,7 +742,7 @@ public class XSDResourceImpl extends ResourceImpl
     String schemaLocation = getURI().toString();
     Collection previouslyAttachedSchemas = attachedSchemas;
     attachedSchemas = null;
-    for (Iterator i = previouslyAttachedSchemas.iterator(); i.hasNext(); )
+    for (Iterator i = previouslyAttachedSchemas.iterator(); i.hasNext();)
     {
       XSDSchema xsdSchema = (XSDSchema)i.next();
       xsdSchema.setSchemaLocation(schemaLocation);
@@ -495,6 +754,28 @@ public class XSDResourceImpl extends ResourceImpl
     }
   }
 
+  /**
+   * Loads a new {@link XSDResourceImpl} into the resource set.
+   * @param inputStream the contents of the new resource.
+   * @param options any options to influence loading behavior.
+   * @return a new XSDResourceImpl.
+   */
+  protected void doLoad(InputStream inputStream, Map options) throws IOException
+  {
+    InputSource inputSource = 
+      inputStream instanceof URIConverter.ReadableInputStream ? 
+      new InputSource(((URIConverter.ReadableInputStream)inputStream).asReader()) :
+      new InputSource(inputStream);
+
+    if (getURI() != null)
+    {
+      String id = getURI().toString();
+      inputSource.setPublicId(id);
+      inputSource.setSystemId(id);
+    }
+    doLoad(inputSource, options);
+  }
+
   protected boolean findSchemas(Element element)
   {
     if (XSDConstants.nodeType(element) == XSDConstants.SCHEMA_ELEMENT)
